"use strict";

var accumulate = require("./accumulate")
var reducible = require("./reducible")
var end = require("./end")
var isError = require("./is-error")

function flatten(source) {
  /**
  Flattens given sequence of sequences to a sequence with items of
  all nested sequences. Note that items in the resulting sequence
  are ordered by the time rather then index, in other words if
  item from the second sub-sequence is deliver earlier then the item
  from first sub-sequence it will in appear earlier in the resulting
  sequence too. If you need to keep order by index you should enforce
  sequential ordering by wrapping `source` into `sequential` before
  passing it to flatten.

  print(flatten([ [1, 2], [ 3, 4 ] ]))  // => <stream 1 2 3 4 />
  **/
  return reducible(function accumulateFlatten(next, initial) {
    var state = initial
    var open = 1

    function forward(value) {
      if (value === end) {
        open = open - 1
        if (open === 0) return next(end)
      } else {
        state = next(value, state)
      }
      return state
    }


    accumulate(source, function accumulateFlattenSource(nested) {
      // If there is an error or end of `source` collection just pass it
      // to `forward` it will take care of detecting weather it's error
      // or `end`. In later case it will also figure out if it's `end` of
      // result to and act appropriately.
      if (nested === end) return forward(end)
      if (isError(nested)) return forward(nested)
      // If `nested` item is not end nor error just `accumulate` it via
      // `forward` that keeps track of all collections that are bing forwarded
      // to it.
      open = open + 1
      accumulate(nested, forward, null)
    }, initial)
  })
}


module.exports = flatten

