(function(){var require = function (file, cwd) {
    var resolved = require.resolve(file, cwd || '/');
    var mod = require.modules[resolved];
    if (!mod) throw new Error(
        'Failed to resolve module ' + file + ', tried ' + resolved
    );
    var cached = require.cache[resolved];
    var res = cached? cached.exports : mod();
    return res;
};

require.paths = [];
require.modules = {};
require.cache = {};
require.extensions = [".js",".coffee"];

require._core = {
    'assert': true,
    'events': true,
    'fs': true,
    'path': true,
    'vm': true
};

require.resolve = (function () {
    return function (x, cwd) {
        if (!cwd) cwd = '/';
        
        if (require._core[x]) return x;
        var path = require.modules.path();
        cwd = path.resolve('/', cwd);
        var y = cwd || '/';
        
        if (x.match(/^(?:\.\.?\/|\/)/)) {
            var m = loadAsFileSync(path.resolve(y, x))
                || loadAsDirectorySync(path.resolve(y, x));
            if (m) return m;
        }
        
        var n = loadNodeModulesSync(x, y);
        if (n) return n;
        
        throw new Error("Cannot find module '" + x + "'");
        
        function loadAsFileSync (x) {
            x = path.normalize(x);
            if (require.modules[x]) {
                return x;
            }
            
            for (var i = 0; i < require.extensions.length; i++) {
                var ext = require.extensions[i];
                if (require.modules[x + ext]) return x + ext;
            }
        }
        
        function loadAsDirectorySync (x) {
            x = x.replace(/\/+$/, '');
            var pkgfile = path.normalize(x + '/package.json');
            if (require.modules[pkgfile]) {
                var pkg = require.modules[pkgfile]();
                var b = pkg.browserify;
                if (typeof b === 'object' && b.main) {
                    var m = loadAsFileSync(path.resolve(x, b.main));
                    if (m) return m;
                }
                else if (typeof b === 'string') {
                    var m = loadAsFileSync(path.resolve(x, b));
                    if (m) return m;
                }
                else if (pkg.main) {
                    var m = loadAsFileSync(path.resolve(x, pkg.main));
                    if (m) return m;
                }
            }
            
            return loadAsFileSync(x + '/index');
        }
        
        function loadNodeModulesSync (x, start) {
            var dirs = nodeModulesPathsSync(start);
            for (var i = 0; i < dirs.length; i++) {
                var dir = dirs[i];
                var m = loadAsFileSync(dir + '/' + x);
                if (m) return m;
                var n = loadAsDirectorySync(dir + '/' + x);
                if (n) return n;
            }
            
            var m = loadAsFileSync(x);
            if (m) return m;
        }
        
        function nodeModulesPathsSync (start) {
            var parts;
            if (start === '/') parts = [ '' ];
            else parts = path.normalize(start).split('/');
            
            var dirs = [];
            for (var i = parts.length - 1; i >= 0; i--) {
                if (parts[i] === 'node_modules') continue;
                var dir = parts.slice(0, i + 1).join('/') + '/node_modules';
                dirs.push(dir);
            }
            
            return dirs;
        }
    };
})();

require.alias = function (from, to) {
    var path = require.modules.path();
    var res = null;
    try {
        res = require.resolve(from + '/package.json', '/');
    }
    catch (err) {
        res = require.resolve(from, '/');
    }
    var basedir = path.dirname(res);
    
    var keys = (Object.keys || function (obj) {
        var res = [];
        for (var key in obj) res.push(key);
        return res;
    })(require.modules);
    
    for (var i = 0; i < keys.length; i++) {
        var key = keys[i];
        if (key.slice(0, basedir.length + 1) === basedir + '/') {
            var f = key.slice(basedir.length);
            require.modules[to + f] = require.modules[basedir + f];
        }
        else if (key === basedir) {
            require.modules[to] = require.modules[basedir];
        }
    }
};

(function () {
    var process = {};
    
    require.define = function (filename, fn) {
        if (require.modules.__browserify_process) {
            process = require.modules.__browserify_process();
        }
        
        var dirname = require._core[filename]
            ? ''
            : require.modules.path().dirname(filename)
        ;
        
        var require_ = function (file) {
            var requiredModule = require(file, dirname);
            var cached = require.cache[require.resolve(file, dirname)];

            if (cached.parent === null) {
                cached.parent = module_;
            }

            return requiredModule;
        };
        require_.resolve = function (name) {
            return require.resolve(name, dirname);
        };
        require_.modules = require.modules;
        require_.define = require.define;
        require_.cache = require.cache;
        var module_ = {
            id : filename,
            filename: filename,
            exports : {},
            loaded : false,
            parent: null
        };
        
        require.modules[filename] = function () {
            require.cache[filename] = module_;
            fn.call(
                module_.exports,
                require_,
                module_,
                module_.exports,
                dirname,
                filename,
                process
            );
            module_.loaded = true;
            return module_.exports;
        };
    };
})();


require.define("path",function(require,module,exports,__dirname,__filename,process){function filter (xs, fn) {
    var res = [];
    for (var i = 0; i < xs.length; i++) {
        if (fn(xs[i], i, xs)) res.push(xs[i]);
    }
    return res;
}

// resolves . and .. elements in a path array with directory names there
// must be no slashes, empty elements, or device names (c:\) in the array
// (so also no leading and trailing slashes - it does not distinguish
// relative and absolute paths)
function normalizeArray(parts, allowAboveRoot) {
  // if the path tries to go above the root, `up` ends up > 0
  var up = 0;
  for (var i = parts.length; i >= 0; i--) {
    var last = parts[i];
    if (last == '.') {
      parts.splice(i, 1);
    } else if (last === '..') {
      parts.splice(i, 1);
      up++;
    } else if (up) {
      parts.splice(i, 1);
      up--;
    }
  }

  // if the path is allowed to go above the root, restore leading ..s
  if (allowAboveRoot) {
    for (; up--; up) {
      parts.unshift('..');
    }
  }

  return parts;
}

// Regex to split a filename into [*, dir, basename, ext]
// posix version
var splitPathRe = /^(.+\/(?!$)|\/)?((?:.+?)?(\.[^.]*)?)$/;

// path.resolve([from ...], to)
// posix version
exports.resolve = function() {
var resolvedPath = '',
    resolvedAbsolute = false;

for (var i = arguments.length; i >= -1 && !resolvedAbsolute; i--) {
  var path = (i >= 0)
      ? arguments[i]
      : process.cwd();

  // Skip empty and invalid entries
  if (typeof path !== 'string' || !path) {
    continue;
  }

  resolvedPath = path + '/' + resolvedPath;
  resolvedAbsolute = path.charAt(0) === '/';
}

// At this point the path should be resolved to a full absolute path, but
// handle relative paths to be safe (might happen when process.cwd() fails)

// Normalize the path
resolvedPath = normalizeArray(filter(resolvedPath.split('/'), function(p) {
    return !!p;
  }), !resolvedAbsolute).join('/');

  return ((resolvedAbsolute ? '/' : '') + resolvedPath) || '.';
};

// path.normalize(path)
// posix version
exports.normalize = function(path) {
var isAbsolute = path.charAt(0) === '/',
    trailingSlash = path.slice(-1) === '/';

// Normalize the path
path = normalizeArray(filter(path.split('/'), function(p) {
    return !!p;
  }), !isAbsolute).join('/');

  if (!path && !isAbsolute) {
    path = '.';
  }
  if (path && trailingSlash) {
    path += '/';
  }
  
  return (isAbsolute ? '/' : '') + path;
};


// posix version
exports.join = function() {
  var paths = Array.prototype.slice.call(arguments, 0);
  return exports.normalize(filter(paths, function(p, index) {
    return p && typeof p === 'string';
  }).join('/'));
};


exports.dirname = function(path) {
  var dir = splitPathRe.exec(path)[1] || '';
  var isWindows = false;
  if (!dir) {
    // No dirname
    return '.';
  } else if (dir.length === 1 ||
      (isWindows && dir.length <= 3 && dir.charAt(1) === ':')) {
    // It is just a slash or a drive letter with a slash
    return dir;
  } else {
    // It is a full dirname, strip trailing slash
    return dir.substring(0, dir.length - 1);
  }
};


exports.basename = function(path, ext) {
  var f = splitPathRe.exec(path)[2] || '';
  // TODO: make this comparison case-insensitive on windows?
  if (ext && f.substr(-1 * ext.length) === ext) {
    f = f.substr(0, f.length - ext.length);
  }
  return f;
};


exports.extname = function(path) {
  return splitPathRe.exec(path)[3] || '';
};
});

require.define("__browserify_process",function(require,module,exports,__dirname,__filename,process){var process = module.exports = {};

process.nextTick = (function () {
    var queue = [];
    var canPost = typeof window !== 'undefined'
        && window.postMessage && window.addEventListener
    ;
    
    if (canPost) {
        window.addEventListener('message', function (ev) {
            if (ev.source === window && ev.data === 'browserify-tick') {
                ev.stopPropagation();
                if (queue.length > 0) {
                    var fn = queue.shift();
                    fn();
                }
            }
        }, true);
    }
    
    return function (fn) {
        if (canPost) {
            queue.push(fn);
            window.postMessage('browserify-tick', '*');
        }
        else setTimeout(fn, 0);
    };
})();

process.title = 'browser';
process.browser = true;
process.env = {};
process.argv = [];

process.binding = function (name) {
    if (name === 'evals') return (require)('vm')
    else throw new Error('No such module. (Possibly not yet loaded)')
};

(function () {
    var cwd = '/';
    var path;
    process.cwd = function () { return cwd };
    process.chdir = function (dir) {
        if (!path) path = require('path');
        cwd = path.resolve(dir, cwd);
    };
})();
});

require.define("/test/filter.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var test = require("./util/test")
var lazy = require("./util/lazy")

var filter = require("../filter")
var into = require("../into")
var concat = require("../concat")
var delay = require("../delay")
var capture = require("../capture")


exports["test filter"] = function(assert) {
  var called = 0
  var source = [ 1, 2, 3 ]
  var actual = filter(source, function(item) {
    called = called + 1
    return item % 2
  })

  assert.equal(called, 0, "filter does not invokes until result is reduced")
  assert.deepEqual(into(actual), [ 1, 3 ], "items were filtered")
  assert.equal(called, 3, "filterer called once per item")
}

exports["test filter empty"] = test(function(assert) {
  var called = 0
  var filtered = filter([], function onEach(element) {
    called = called + 1
  })
  var actual = concat(filtered,
                      lazy(function() { return called }))

  assert(actual, [0], "filter `f` was not executed")
})

exports["test filter numbers"] = test(function(assert) {
  var called = 0
  var filtered = filter([1, 2, 3, 4], function(n) {
    called = called + 1
    return n % 2
  })
  var actual = concat(filtered,
                      lazy(function() { return called }))

  assert(actual, [1, 3, 4], "filter `f` was called once per element")
})

exports["test filter async stream"] = test(function(assert) {
  var called = 0
  var filtered = filter(delay([5, 4, 3, 2, 1]), function(n) {
    called = called + 1
    return n % 2
  })
  var actual = concat(filtered,
                      lazy(function() { return called }))

  assert(actual, [5, 3, 1, 5], "predicate was called once per element")
})

exports["test errors propagate"] = test(function(assert) {
  var boom = Error("Boom!")
  var filtered = filter(delay(concat([3, 2, 1], boom)), function(n) {
    return n % 2
  })
  var actual = capture(filtered, function(e) { return e.message })

  assert(actual, [3, 1, boom.message], "error do propagate")
})

if (require.main === module)
  require("test").run(exports)
});

require.define("/test/util/test.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var accumulate = require("../../accumulate")
var isError = require("../../is-error")
var end = require("../../end")
var isReduced = require("../../is-reduced")

function test(unit) {
  return function(assertions, done) {
    function assert(actual, expected, comment) {
      var values = []
      accumulate(actual, function(actual) {
        if (actual === end) {
          assert.deepEqual(values, expected, comment)
          done()
        } else if (isError(actual)) {
          assert.deepEqual({ values: values, error: actual }, expected, comment)
          done()
        } else if (isReduced(actual)) {
          return actual
        } else {
          values.push(actual)
        }
        return actual
      })
    }

    for (var key in assertions) assert[key] = assertions[key].bind(assertions)

    unit(assert)
  }
}

module.exports = test
});

require.define("/package.json",function(require,module,exports,__dirname,__filename,process){module.exports = {}});

require.define("/accumulate.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var Method = require("method")

var isReduced = require("./is-reduced")
var isError = require("./is-error")
var end = require("./end")

var Eventual = require("eventual/type")
var when = require("eventual/when")

var accumulate = Method()

// Implementation of accumulate for the empty sequences, that immediately
// signals end of sequence.
accumulate.empty = function accumulateEmpty(empty, next, initial) {
  next(end, initial)
}
// Implementation of accumulate for the singular values which are treated
// as sequences with single element. Yields the given value and signals
// the end of sequence.
accumulate.singular = function accumulateSingular(value, next, initial) {
  next(end, next(value, initial))
}

// Implementation accumulate for the array (and alike) values, such that it
// will call accumulator function `next` each time with next item and
// accumulated state until it's exhausted or `next` returns marked value
// indicating that it's done accumulating. Either way signals end to
// an accumulator.
accumulate.indexed = function accumulateIndexed(indexed, next, initial) {
  var state = initial
  var index = 0
  var count = indexed.length
  while (index < count) {
    var value = indexed[index]
    state = next(value, state)
    index = index + 1
    if (value === end) return end
    if (isError(value)) return state
    if (isReduced(state)) return state.value
  }
  next(end, state)
}

// Both `undefined` and `null` implement accumulate for empty sequences.
accumulate.define(void(0), accumulate.empty)
accumulate.define(null, accumulate.empty)

// Array and arguments implement accumulate for indexed sequences.
accumulate.define(Array, accumulate.indexed)
accumulate.define((function() { return arguments })(), accumulate.indexed)

// All other built-in data structures are treated as single value sequences
// by default. Of course individual types may choose to override that.
accumulate.define(accumulate.singular)

// All eventual values are treated as a single value of sequences, of
// the value they realize to.
accumulate.define(Eventual, function(eventual, next, initial) {
  return when(eventual, function delivered(value) {
    return accumulate(value, next, initial)
  }, function failed(error) {
    next(error, initial)
    return error
  })
})

module.exports = accumulate
});

require.define("/node_modules/method/package.json",function(require,module,exports,__dirname,__filename,process){module.exports = {"main":"./core.js"}});

require.define("/node_modules/method/core.js",function(require,module,exports,__dirname,__filename,process){/* vim:set ts=2 sw=2 sts=2 expandtab */
/*jshint asi: true undef: true es5: true node: true browser: true devel: true
         forin: true latedef: false globalstrict: true */
'use strict';

// Shortcuts for ES5 reflection functions.
var make = Object.create || (function() {
  var Type = function Type() {}
  return function make(prototype) {
    Type.prototype = prototype
    return new Type()
  }
})
var defineProperty = Object.defineProperty || function(object, name, property) {
  object[name] = property.value
  return object
}
var typefy = Object.prototype.toString

function Method(base) {
  /**
  Private Method is a callable private name that dispatches on the first
  arguments same named Method: Method(...rest) => rest[0][Method](...rest)
  Default implementation may be passed in as an argument.
  **/

  // Create an internal unique name if default implementation is passed,
  // use it's name as a name hint.
  var name = (base && base.name || "") + Math.random().toString(32).substr(2)

  function dispatch() {
    // Method dispatches on type of the first argument.
    var target = arguments[0]
    var builtin = null
    // If first argument is `null` or `undefined` use associated property
    // maps for implementation lookups, otherwise attempt to use implementation
    // for built-in falling back for implementation on the first argument.
    // Finally use default implementation if no other one is found.
    var method = target === null ? Null[name] :
                 target === undefined ? Undefined[name] :
                 target[name] ||
                 ((builtin = Builtins[typefy.call(target)]) && builtin[name]) ||
                 Builtins.Object[name] ||
                 Default[name]

    // If implementation not found there's not much we can do about it,
    // throw error with a descriptive message.
    if (!method) throw Error('Type does not implements method')

    // If implementation is found delegate to it.
    return method.apply(method, arguments)
  }

  // Define default implementation.
  Default[name] = base

  // Define `Method.toString` returning private name, this hack will enable
  // Method definition like follows:
  // var foo = Method()
  // object[foo] = function() { /***/ }
  dispatch.toString = function toString() { return name }

  // Copy utility Methods for convenient API.
  dispatch.implement = implementMethod
  dispatch.define = defineMethod

  return dispatch
}

// Define objects where Methods implementations for `null`, `undefined` and
// defaults will be stored.
var Default = {}
var Null = make(Default)
var Undefined = make(Default)
// Implementation for built-in types are stored in the hash, this avoids
// mutations on built-ins and allows cross frame extensions. Primitive types
// are predefined so that `Object` extensions won't be inherited.
var Builtins = {
  Object: make(Default),
  Number: make(Default),
  String: make(Default),
  Boolean: make(Default)
}
// Define aliases for predefined built-in maps to a forms that values will
// be serialized on dispatch.
Builtins[typefy.call(Object.prototype)] = Builtins.Object
Builtins[typefy.call(Number.prototype)] = Builtins.Number
Builtins[typefy.call(String.prototype)] = Builtins.String



var implement = Method(
function implement(method, object, lambda) {
  /**
  Implements `Method` for the given `object` with a provided `implementation`.
  Calling `Method` with `object` as a first argument will dispatch on provided
  implementation.
  **/
  var target = object === null ? Null :
               object === undefined ? Undefined :
               object

  return defineProperty(target, method.toString(), {
    enumerable: false,
    configurable: false,
    writable: false,
    value: lambda
  })
})

var define = Method(function define(method, Type, lambda) {
  /**
  Defines `Method` for the given `Type` with a provided `implementation`.
  Calling `Method` with a first argument of this `Type` will dispatch on
  provided `implementation`. If `Type` is a `Method` default implementation
  is defined. If `Type` is a `null` or `undefined` `Method` is implemented
  for that value type.
  **/
  if (!lambda) return implement(method, Default, Type)
  if (!Type) return implement(method, Type, lambda)
  var type = typefy.call(Type.prototype)
  return type !== "[object Object]" ? implement(method,
      Builtins[type] || (Builtins[type] = make(Builtins.Object)), lambda) :
    // This should be `Type === Object` but since it will be `false` for
    // `Object` from different JS context / compartment / frame we assume that
    // if it's name is `Object` it is Object.
    Type.name === "Object" ? implement(method, Builtins.Object, lambda) :
    implement(method, Type.prototype, lambda)
})

var defineMethod = function defineMethod(Type, lambda) {
  return define(this, Type, lambda)
}
var implementMethod = function implementMethod(object, lambda) {
  return implement(this, object, lambda)
}


// Define exports on `Method` as it's only thing we export.
Method.implement = implement
Method.define = define
Method.Method = Method
Method.Null = Null
Method.Undefined = Undefined
Method.Default = Default
Method.Builtins = Builtins

module.exports = Method
});

require.define("/is-reduced.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var reduced = require("./reduced")

function isReduced(value) {
  return value && value.is === reduced
}

module.exports = isReduced
});

require.define("/reduced.js",function(require,module,exports,__dirname,__filename,process){"use strict";


// Exported function can be used for boxing values. This boxing indicates
// that consumer of sequence has finished consuming it, there for new values
// should not be no longer pushed.
function reduced(value) {
  /**
  Boxes given value and indicates to a source that it's already reduced and
  no new values should be supplied
  **/
  return { value: value, is: reduced }
}

module.exports = reduced
});

require.define("/is-error.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var stringifier = Object.prototype.toString

function isError(value) {
  return stringifier.call(value) === "[object Error]"
}

module.exports = isError
});

require.define("/end.js",function(require,module,exports,__dirname,__filename,process){"use strict";

module.exports = String("End of the collection")
});

require.define("/node_modules/eventual/package.json",function(require,module,exports,__dirname,__filename,process){module.exports = {}});

require.define("/node_modules/eventual/type.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var watchers = require("watchables/watchers")
var watch = require("watchables/watch")
var await = require("pending/await")
var isPending = require("pending/is")
var deliver = require("./deliver")
var when = require("./when")

// Internal utility function returns true if given value is of error type,
// otherwise returns false.
var isError = (function() {
  var stringy = Object.prototype.toString
  var error = stringy.call(Error.prototype)
  return function isError(value) {
    return stringy.call(value) === error
  }
})()

// Internal utility, identity function. Returns whatever is given to it.
function identity(value) { return value }

// Internal utility, decorator function that wraps given function into
// try / catch and returns thrown exception in case when exception is
// thrown.
function attempt(f) {
  return function effort(value) {
    try { return f(value) }
    catch (error) { return error }
  }
}


// Define property names used by an `Eventual` type. Names are prefixed via
// `module.id` to avoid name conflicts.
var observers = "observers@" + module.id
var result = "value@" + module.id
var pending = "pending@" + module.id


function Eventual() {
  /**
  Data type representing eventual value, that can be observed and delivered.
  Type implements `watchable`, `pending` and `eventual` abstractions, where
  first two are defined in an external libraries.
  **/
  this[observers] = []
  this[result] = this
  this[pending] = true
}
// Expose property names via type static properties so that it's easier
// to refer to them while debugging.
Eventual.observers = observers
Eventual.result = result
Eventual.pending = pending

watchers.define(Eventual, function(value) {
  return value[observers]
})
// Eventual values considered to be pending until the are deliver by calling
// `deliver`. Internal `pending` property is used to identify weather value
// is being watched or not.
isPending.define(Eventual, function(value) {
  return value[pending]
})
// Eventual type implements await function of pending abstraction, to enable
// observation of value delivery.
await.define(Eventual, function(value, observer) {
  if (isPending(value)) watch(value, observer)
  else observer(value[result])
})

// Eventual implements `deliver` function of pending abstraction, to enable
// fulfillment of eventual values. Eventual value can be delivered only once,
// which will transition it from pending state to non-pending state. All
// further deliveries are ignored. It's also guaranteed that all the registered
// observers will be invoked in FIFO order.
deliver.define(Eventual, function(value, data) {
  // Ignore delivery if value is no longer pending, or if it's in a process of
  // delivery (in this case eventual[result] is set to value other than
  // eventual itself). Also ignore if data deliver is value itself.
  if (value !== data && isPending(value) && value[result] === value) {
    var count = 0
    var index = 0
    var delivering = true
    var observers = void(0)
    // Set eventual value result to passed data value that also marks value
    // as delivery in progress. This way all the `deliver` calls is side
    // effect to this will be ignored. Note: value should still remain pending
    // so that new observers could be registered instead of being called
    // immediately, otherwise it breaks FIFO order.
    value[result] = data
    while (delivering) {
      // If current batch of observers is exhausted, splice a new batch
      // and continue delivery. New batch is created only if new observers
      // are registered in side effect to this call of deliver.
      if (index === count) {
        observers = watchers(value).splice(0)
        count = observers.length
        index = 0
        // If new observers have not being registered mark value as no longer
        // pending and finish delivering.
        if (count === index) {
          value[pending] = false
          delivering = false
        }
      }
      // Register await handlers on given result, is it may be eventual /
      // pending itself. Delivering eventual will cause delivery of the
      // delivered eventual's delivery value, whenever that would be.
      else {
        await(data, observers[index])
        index = index + 1
      }
    }
  }
})

// Eventual implements `when` polymorphic function that is part of it's own
// abstraction. It takes `value` `onFulfill` & `onError` handlers. In return
// when returns eventual value, that is delivered return value of the handler
// that is invoked depending on the given values delivery. If deliver value
// is of error type error handler is invoked. If value is delivered with other
// non-pending value that is not of error type `onFulfill` handlers is invoked
// with it. If pending value is delivered then it's value will be delivered
// it's result whenever that would be. This will cause both value and error
// propagation.
when.define(Eventual, function(value, onRealize, onError) {
  // Create eventual value for a return value.
  var delivered = false
  var eventual = void(0)
  var result = void(0)
  // Wrap handlers into attempt decorator function, so that in case of
  // exception thrown error is returned causing error propagation. If handler
  // is missing identity function is used instead to propagate value / error.
  var realize = onRealize ? attempt(onRealize) : identity
  var error = onError ? attempt(onError) : identity
  // Wait for pending value to be delivered.
  await(value, function onDeliver(data) {
    // Once value is delivered invoke appropriate handler, and deliver it
    // to a resulting eventual value.
    result = isError(data) ? error(data)
                           : realize(data)

    // If outer function is already returned and has created eventual
    // for it's result deliver it. Otherwise (if await called observer
    // in same synchronously) mark result delivered.
    if (eventual) deliver(eventual, result)
    else delivered = true
  })

  // If result is delivered already return it, otherwise create eventual
  // value for the result and return that.
  return delivered ? result : (eventual = new Eventual())
})

module.exports = Eventual
});

require.define("/node_modules/eventual/node_modules/watchables/package.json",function(require,module,exports,__dirname,__filename,process){module.exports = {}});

require.define("/node_modules/eventual/node_modules/watchables/watchers.js",function(require,module,exports,__dirname,__filename,process){/*jshint asi: true undef: true es5: true node: true devel: true
         globalstrict: true forin: true latedef: false supernew: true */
/*global define: true */

"use strict";

var Method = require('method')

// Method is supposed to return array of watchers for the given
// value.
module.exports = Method()
});

require.define("/node_modules/eventual/node_modules/watchables/watch.js",function(require,module,exports,__dirname,__filename,process){/*jshint asi: true undef: true es5: true node: true devel: true
         globalstrict: true forin: true latedef: false supernew: true */
/*global define: true */

"use strict";

var Method = require('method')
var watchers = require('./watchers')

module.exports = Method(function(value, watcher) {
  // Registers a `value` `watcher`, unless it's already registered.
  var registered = watchers(value)
  if (registered && registered.indexOf(watcher) < 0)
    registered.push(watcher)
  return value
})
});

require.define("/node_modules/eventual/node_modules/pending/package.json",function(require,module,exports,__dirname,__filename,process){module.exports = {}});

require.define("/node_modules/eventual/node_modules/pending/await.js",function(require,module,exports,__dirname,__filename,process){/*jshint asi: true undef: true es5: true node: true devel: true
         globalstrict: true forin: true latedef: false supernew: true */
/*global define: true */

"use strict";

var Method = require('method')

// Set's up a callback to be called once pending
// value is realized. All object by default are realized.
module.exports = Method(function(value, callback) {
  callback(value)
})
});

require.define("/node_modules/eventual/node_modules/pending/is.js",function(require,module,exports,__dirname,__filename,process){/*jshint asi: true undef: true es5: true node: true devel: true
         globalstrict: true forin: true latedef: false supernew: true */
/*global define: true */

"use strict";

var Method = require('method')

// Returns `true` if given `value` is pending, otherwise returns
// `false`. All types will return false unless type specific
// implementation is provided to do it otherwise.
module.exports = Method(function() {
  return false
})
});

require.define("/node_modules/eventual/deliver.js",function(require,module,exports,__dirname,__filename,process){"use strict";

// Anyone crating an eventual will likely need to realize it, requiring
// dependency on other package is complicated, not to mention that one
// can easily wind up with several copies that does not necessary play
// well with each other. Exposing this solves the issues.
module.exports = require("pending/deliver")
});

require.define("/node_modules/eventual/node_modules/pending/deliver.js",function(require,module,exports,__dirname,__filename,process){/*jshint asi: true undef: true es5: true node: true devel: true
         globalstrict: true forin: true latedef: false supernew: true */
/*global define: true */

"use strict";

var Method = require('method')
// Method delivers pending value.
module.exports = Method()
});

require.define("/node_modules/eventual/when.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var Method = require("method")
var when = Method(function(value, onRealize) {
  return typeof(onRealize) === "function" ? onRealize(value) : value
})
when.define(Error, function(error, onRealize, onError) {
  return typeof(onError) === "function" ? onError(error) : error
})

module.exports = when
});

require.define("/test/util/lazy.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var map = require("../../map")

function lazy(f) {
  return map(1, function() { return f() })
}

module.exports = lazy
});

require.define("/map.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var reducer = require("./reducer")

var map = reducer(function map(f, next, value, result) {
  /**
  Returns transformed version of given `source` where each item of it
  is mapped using `f`.

  ## Example

  var data = [{ name: "foo" }, { name: "bar" }]
  var names = map(data, function(value) { return value.name })
  print(names) // => < "foo" "bar" >
  **/
  next(f(value), result)
})

module.exports = map
});

require.define("/reducer.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var accumulate = require("./accumulate")
var reducible = require("./reducible")
var isError = require("./is-error")
var end = require("./end")


function reducer(process) {
  return function reducer(source, options) {
    return reducible(function(next, initial) {
      accumulate(source, function(value, result) {
        return value === end ? next(value, result) :
               isError(value) ? next(value, result) :
               process(options, next, value, result)
      })
    })
  }
}

module.exports = reducer
});

require.define("/reducible.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var accumulate = require("./accumulate")
var when = require("eventual/when")
var end = require("./end")
var isError = require("./is-error")
var isReduced = require("./is-reduced")
var reduced = require("./reduced")

function Reducible(reduce) {
  /**
  Reducible is a type of the data-structure that represents something
  that can be reduced. Most of the time it's used to represent transformation
  over other reducible by capturing it in a lexical scope.

  Reducible has an attribute `reduce` pointing to a function that does
  reduction.
  **/

  // JS engines optimize access to properties that are set in the constructor's
  // so we set it here.
  this.reduce = reduce
}

// Implementation of `accumulate` for reducible, which just delegates to it's
// `reduce` attribute.
accumulate.define(Reducible, function accumulate(reducible, next, initial) {
  var result
  // State is intentionally accumulated in the outer variable, that way no
  // matter if consumer is broken and passes in wrong accumulated state back
  // this reducible will still behave as intended.
  var state = initial
  reducible.reduce(function forward(value) {
    try {
      // If reduce reduction has already being completed just return
      // `result` that is last state boxed in `reduced`. That way anything
      // trying to dispatch after it's closed or error-ed will just be handed
      // a `reduced` `state` indicating last value and no intent of getting
      // more values.
      if (result) return result

      // If value is an `error` (that also includes `end` of stream) we just
      // throw and let `catch` block do the rest of the job.
      if (value === end || isError(value)) throw value

      // Otherwise new `state` is accumulated `by` forwarding a `value` to an
      // actual `next` handler.
      state = next(value, state)

      // If new `state` is boxed in `reduced` than source should be stopped
      // and no more values should be forwarded to a `next` handler. To do
      // that we throw `end` and let `catch` block do the rest of the job.
      if (isReduced(state)) throw end

      // If code got that far then nothing special happened and `new` state is
      // just returned back, to a consumer.
      return state
    }
    // If `error` is thrown that may few things:
    //
    //  1. Last value dispatched was indicator of an error (that also includes
    //     `end` of stream).
    //  2. Provided `next` handler threw an exception causing stream failure.
    //
    // When this happens stream is either finished or error-ed, either way
    // no new items should get through. There for last `state` is boxed with
    // `reduced` and store as a `result` of this accumulation. Any subsequent
    // attempts of providing values will just get it in return, hopefully
    // causing source of value to get closed.
    catch (error) {
      if (isReduced(state)) {
        result = state
        state = result.value
      } else {
        result = reduced(state)
      }
      // Maybe we should console.error exceptions if such arise when calling
      // `next` in the following line.
      next(error, state)
      return result
    }
  }, null)
})

function reducible(reduce) {
  return new Reducible(reduce)
}
reducible.type = Reducible

module.exports = reducible
});

require.define("/filter.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var reducer = require("./reducer")

var filter = reducer(function filter(predicate, next, value, result) {
  /**
  Composes filtered version of given `source`, such that only items contained
  will be once on which `f(item)` was `true`.

  ## Example

  var digits = filter([ 10, 23, 2, 7, 17 ], function(value) {
    return value >= 0 && value <= 9
  })
  print(digits) // => < 2 7 >
  **/
  return predicate(value) ? next(value, result) :
         result
})

module.exports = filter
});

require.define("/into.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var reduce = require("./reduce")

function into(source, buffer) {
  /**
  Adds items of given `reducible` into
  given `array` or a new empty one if omitted.
  **/
  return reduce(source, function reduceInto(result, value) {
    result.push(value)
    return result
  }, buffer || [])
}

module.exports = into
});

require.define("/reduce.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var deliver = require("eventual/deliver")
var defer = require("eventual/defer")
var when = require("eventual/when")
var accumulate = require("./accumulate")
var isError = require("./is-error")
var isReduced = require("./is-reduced")
var end = require("./end")

function reduce(source, next, initial) {
  var promise = defer()
  accumulate(source, function reduce(value, state) {

    if (value === end) return deliver(promise, state)
    else if (isError(value)) return deliver(promise, value)

    state = next(state, value)

    if (isReduced(state)) deliver(promise, state.value)

    return state
  }, initial)
  return when(promise, null, function(error) { return error })
}

module.exports = reduce
});

require.define("/node_modules/eventual/defer.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var Eventual = require("./type")
module.exports = function defer() { return new Eventual() }
});

require.define("/concat.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var reducible = require("./reducible")
var accumulate = require("./accumulate")
var end = require("./end")

var slicer = Array.prototype.slice

function append(left, right) {
  /**
  Returns sequences of items in the `left` sequence followed by the
  items in the `right` sequence.
  **/
  return reducible(function(next, initial) {
    accumulate(left, function(value, result) {
      return value === end ? accumulate(right, next, result) :
             next(value, result)
    }, initial)
  })
}

function concat(left, right /*, ...rest*/) {
  /**
  Returns a sequence representing the concatenation of the elements in the
  supplied arguments, in the given order.

  print(concat([ 1 ], [ 2, 3 ], [ 4, 5, 6 ])) // => <stream 1 2 3 4 5 6 />

  **/
  switch (arguments.length) {
    case 1: return left
    case 2: return append(left, right)
    default: return slicer.call(arguments).reduce(append)
  }
}

module.exports = concat
});

require.define("/delay.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var reducible = require("./reducible")
var accumulate = require("./accumulate")
var isReduced = require("./is-reduced")
var end = require("./end")

function delay(source, ms) {
  ms = ms || 6 // Minimum 6ms, as on less dispatch order becomes unreliable
  return reducible(function(next, result) {
    var timeout = 0
    var ended = false
    accumulate(source, function(value) {
      setTimeout(function() {
        if (!ended) {
          timeout = timeout - ms
          result = next(value, result)
          if (isReduced(result)) {
            ended = true
            next(end)
          }
        }
      }, timeout = timeout + ms)
      return result
    })
  })
}

module.exports = delay
});

require.define("/capture.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var reducible = require("./reducible")
var accumulate = require("./accumulate")
var reduced = require("./reduced")
var isError = require("./is-error")

function capture(source, recover) {
  /**
  Creates and returns safe version of given `source` sequence, by using
  `recover` function to recover from errors that may occur while reducing
  a `source`. This is a mechanism for error handling and recovery for streams
  that representing IO operations like (XHR / WebSockets etc...) where errors
  may occur.
  **/
  return reducible(function(next, initial) {
    var failure = void(0)
    accumulate(source, function(value, result) {
      // If error has already being captured then return
      if (failure) return failure
      // If value is an error then continue accumulation of recovered
      // sequence.
      else if (isError(value)) {
        failure = reduced(result)
        accumulate(recover(value, result), next, result)
        return failure
      }
      // Otherwise just forward messages.
      else return next(value, result)
    }, initial)
  })
}

module.exports = capture
});

require.define("/node_modules/test/package.json",function(require,module,exports,__dirname,__filename,process){module.exports = {"main":"./test.js"}});

require.define("/node_modules/test/test.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var Assert = require("./assert").Assert
var Logger = require("./logger").Logger


var ERR_COMPLETED_ASSERT = "Assert in completed test"
var ERR_COMPLETED_COMPLETE = "Attemt to complete test more then one times"
var ERR_EXPECT = "AssertionError"


/**
 * Creates a test function.
 */
function Test(name, unit, logger, Assert) {
  var isSync = unit.length <= 1
  var isFailFast = !unit.length
  var isDone = false
  return function test(next) {
    logger = logger.section(name)
    var assert = Assert(logger)
    assert.end = function end() {
      if (isDone) return logger.error(Error(ERR_COMPLETED_COMPLETE))
      isDone = true
      next()
    }

    try {
      var result = unit(assert, assert.end)
      // If it"s async test that returns a promise.
      if (result && typeof(result.then) === "function") {
        result.then(function passed() {
          logger.pass("passed")
          assert.end()
        }, function failed(reason) {
          logger.fail(reason)
          assert.end()
        })
      } else {
        if (isFailFast) logger.pass("passed")
        if (isSync) assert.end()
      }
    } catch (exception) {
      if (ERR_EXPECT === exception.name) assert.fail(exception)
      else logger.error(exception)
      assert.end()
    }
  }
}

function isTest(name) { return name.indexOf("test") === 0 }

/**
 * Creates a test suite / group. Calling returned function will execute
 * all test in the given suite.
 */
function Suite(name, units, logger, Assert) {
  // Collecting properties that represent test functions or suits.
  var names = Object.keys(units).filter(isTest)
  Assert = units.Assert || Assert
  // Returning a function that executes all test in this suite and all it"s
  // sub-suits.
  return function suite(end) {
    // Chaining test / suits so that each is executed after last is done.
    function next() {
      if (!names.length) return end()
      var name = names.shift()
      var unit = Unit(name, units[name], logger, units.Assert || Assert)
      unit(next)
    }
    next((logger = logger.section(name)))
  }
}
function Unit(name, units, logger, Assert) {
  return typeof(units) === "function" ? Test(name, units, logger, Assert)
                                      : Suite(name, units, logger, Assert)
}


/**
 * Test runner function.
 */
exports.run = function run(units, logger) {
  var exit = logger ? false : true
  logger = logger || new Logger()
  var unit = Unit("Running all tests:", units, logger, Assert)
  unit(function done() {
    logger.report()
    var failed = logger.errors.length !== 0 || logger.fails.length !== 0
    // Exit only if `process.exit` exist and if no logger was provided.
    if (exit && process.exit) process.exit(failed ? 1 : 0)
  })
}
});

require.define("/node_modules/test/assert.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var utils = require("./utils")


/**
 * The `AssertionError` is defined in assert.
 * @extends Error
 * @example
 *  new assert.AssertionError({
 *    message: message,
 *    actual: actual,
 *    expected: expected
 *  })
 */
function AssertionError(options) {
  var assertionError = Object.create(AssertionError.prototype);

  if (utils.isString(options))
    options = { message: options };
  if ("actual" in options)
    assertionError.actual = options.actual;
  if ("expected" in options)
    assertionError.expected = options.expected;
  if ("operator" in options)
    assertionError.operator = options.operator;

  assertionError.message = options.message;
  assertionError.stack = new Error().stack;
  return assertionError;
}
AssertionError.prototype = Object.create(Error.prototype, {
  constructor: { value: AssertionError },
  name: { value: "AssertionError", enumerable: true },
  toString: { value: function toString() {
    var value;
    if (this.message) {
      value = this.name + " : " + this.message;
    }
    else {
      value = [
        this.name + " : ",
        utils.source(this.expected),
        this.operator,
        utils.source(this.actual)
      ].join(" ");
    }
    return value;
  }}
});
exports.AssertionError = AssertionError;

function Assert(logger) {
  return Object.create(Assert.prototype, { _log: { value: logger }});
}
Assert.prototype = {
  fail: function fail(e) {
    this._log.fail(e);
  },
  pass: function pass(message) {
    this._log.pass(message);
  },
  error: function error(e) {
    this._log.error(e);
  },
  ok: function ok(value, message) {
    if (!!!value) {
      this.fail({
        actual: value,
        expected: true,
        message: message,
        operator: "=="
      });
    }
    else {
      this.pass(message);
    }
  },

  /**
   * The equality assertion tests shallow, coercive equality with `==`.
   * @example
   *    assert.equal(1, 1, "one is one");
   */
  equal: function equal(actual, expected, message) {
    if (actual == expected) {
      this.pass(message);
    }
    else {
      this.fail({
        actual: actual,
        expected: expected,
        message: message,
        operator: "=="
      });
    }
  },

  /**
   * The non-equality assertion tests for whether two objects are not equal
   * with `!=`.
   * @example
   *    assert.notEqual(1, 2, "one is not two");
   */
  notEqual: function notEqual(actual, expected, message) {
    if (actual != expected) {
      this.pass(message);
    }
    else {
      this.fail({
        actual: actual,
        expected: expected,
        message: message,
        operator: "!=",
      });
    }
  },

  /**
   * The equivalence assertion tests a deep (with `===`) equality relation.
   * @example
   *    assert.deepEqual({ a: "foo" }, { a: "foo" }, "equivalent objects")
   */
   deepEqual: function deepEqual(actual, expected, message) {
    if (isDeepEqual(actual, expected)) {
      this.pass(message);
    }
    else {
      this.fail({
        actual: actual,
        expected: expected,
        message: message,
        operator: "deepEqual"
      });
    }
  },

  /**
   * The non-equivalence assertion tests for any deep (with `===`) inequality.
   * @example
   *    assert.notDeepEqual({ a: "foo" }, Object.create({ a: "foo" }),
   *                        "object"s inherit from different prototypes");
   */
  notDeepEqual: function notDeepEqual(actual, expected, message) {
    if (!isDeepEqual(actual, expected)) {
      this.pass(message);
    }
    else {
      this.fail({
        actual: actual,
        expected: expected,
        message: message,
        operator: "notDeepEqual"
      });
    }
  },

  /**
   * The strict equality assertion tests strict equality, as determined by
   * `===`.
   * @example
   *    assert.strictEqual(null, null, "`null` is `null`")
   */
  strictEqual: function strictEqual(actual, expected, message) {
    if (actual === expected) {
      this.pass(message);
    }
    else {
      this.fail({
        actual: actual,
        expected: expected,
        message: message,
        operator: "==="
      });
    }
  },

  /**
   * The strict non-equality assertion tests for strict inequality, as
   * determined by `!==`.
   * @example
   *    assert.notStrictEqual(null, undefined, "`null` is not `undefined`");
   */
  notStrictEqual: function notStrictEqual(actual, expected, message) {
    if (actual !== expected) {
      this.pass(message);
    }
    else {
      this.fail({
        actual: actual,
        expected: expected,
        message: message,
        operator: "!=="
      })
    }
  },

  /**
   * The assertion whether or not given `block` throws an exception. If optional
   * `Error` argument is provided and it"s type of function thrown error is
   * asserted to be an instance of it, if type of `Error` is string then message
   * of throw exception is asserted to contain it.
   * @param {Function} block
   *    Function that is expected to throw.
   * @param {Error|RegExp} [Error]
   *    Error constructor that is expected to be thrown or a string that
   *    must be contained by a message of the thrown exception, or a RegExp
   *    matching a message of the thrown exception.
   * @param {String} message
   *    Description message
   *
   * @examples
   *
   *    assert.throws(function block() {
   *      doSomething(4)
   *    }, "Object is expected", "Incorrect argument is passed");
   *
   *    assert.throws(function block() {
   *      Object.create(5)
   *    }, TypeError, "TypeError is thrown");
   */
  throws: function throws(block, Error, message) {
    var threw = false;
    var exception = null;

    // If third argument is not provided and second argument is a string it
    // means that optional `Error` argument was not passed, so we shift
    // arguments.
    if (utils.isString(Error) && utils.isUndefined(message)) {
      message = Error;
      Error = undefined;
    }

    // Executing given `block`.
    try {
      block();
    }
    catch (e) {
      threw = true;
      exception = e;
    }

    // If exception was thrown and `Error` argument was not passed assert is
    // passed.
    if (threw && (utils.isUndefined(Error) ||
                 // If Error is thrown exception
                 (Error == exception) ||
                 // If passed `Error` is RegExp using it"s test method to
                 // assert thrown exception message.
                 (utils.isRegExp(Error) && Error.test(exception.message)) ||
                 // If passed `Error` is a constructor function testing if
                 // thrown exception is an instance of it.
                 (utils.isFunction(Error) && utils.instanceOf(exception, Error))))
    {
      this.pass(message);
    }

    // Otherwise we report assertion failure.
    else {
      var failure = {
        message: message,
        operator: "throws"
      };

      if (exception)
        failure.actual = exception;

      if (Error)
        failure.expected = Error;

      this.fail(failure);
    }
  }
};
exports.Assert = Assert;

function isDeepEqual(actual, expected) {
  // 7.1. All identical values are equivalent, as determined by ===.
  if (actual === expected) {
    return true;
  }

  // 7.2. If the expected value is a Date object, the actual value is
  // equivalent if it is also a Date object that refers to the same time.
  else if (utils.isDate(actual) && utils.isDate(expected)) {
    return actual.getTime() === expected.getTime();
  }

  // XXX specification bug: this should be specified
  else if (utils.isPrimitive(actual) || utils.isPrimitive(expected)) {
    return expected === actual;
  }

  // 7.3. Other pairs that do not both pass typeof value == "object",
  // equivalence is determined by ==.
  else if (!utils.isObject(actual) && !utils.isObject(expected)) {
    return actual == expected;
  }

  // 7.4. For all other Object pairs, including Array objects, equivalence is
  // determined by having the same number of owned properties (as verified
  // with Object.prototype.hasOwnProperty.call), the same set of keys
  // (although not necessarily the same order), equivalent values for every
  // corresponding key, and an identical "prototype" property. Note: this
  // accounts for both named and indexed properties on Arrays.
  else {
    return actual.prototype === expected.prototype &&
           isEquivalent(actual, expected);
  }
}

function isEquivalent(a, b, stack) {
  return isArrayEquivalent(Object.keys(a).sort(),
                           Object.keys(b).sort()) &&
          Object.keys(a).every(function(key) {
            return isDeepEqual(a[key], b[key], stack)
          });
}

function isArrayEquivalent(a, b, stack) {
  return utils.isArray(a) && utils.isArray(b) && a.length === b.length &&
         a.every(function(value, index) {
           return isDeepEqual(value, b[index]);
         });
}
});

require.define("/node_modules/test/utils.js",function(require,module,exports,__dirname,__filename,process){"use strict";

/**
 * Returns `true` if `value` is `undefined`.
 * @examples
 *    var foo; isUndefined(foo); // true
 *    isUndefined(0); // false
 */
function isUndefined(value) {
  return value === undefined;
}
exports.isUndefined = isUndefined;

/**
 * Returns `true` if value is `null`.
 * @examples
 *    isNull(null); // true
 *    isNull(undefined); // false
 */
function isNull(value) {
  return value === null;
}
exports.isNull = isNull;

/**
 * Returns `true` if value is a string.
 * @examples
 *    isString("moe"); // true
 */
function isString(value) {
  return typeof value === "string";
}
exports.isString = isString;

/**
 * Returns `true` if `value` is a number.
 * @examples
 *    isNumber(8.4 * 5); // true
 */
function isNumber(value) {
  return typeof value === "number";
}
exports.isNumber = isNumber;

/**
 * Returns `true` if `value` is a `RegExp`.
 * @examples
 *    isRegExp(/moe/); // true
 */
function isRegExp(value) {
  return instanceOf(value, RegExp);
}
exports.isRegExp = isRegExp;

/**
 * Returns true if `value` is a `Date`.
 * @examples
 *    isDate(new Date()); // true
 */
function isDate(value) {
  return isObject(value) && instanceOf(value, Date);
}
exports.isDate = isDate;

/**
 * Returns true if object is a Function.
 * @examples
 *    isFunction(function foo(){}) // true
 */
function isFunction(value) {
    return typeof value === "function" && value.call && value.apply;
}
exports.isFunction = isFunction;

/**
 * Returns `true` if `value` is an object (please note that `null` is considered
 * to be an atom and not an object).
 * @examples
 *    isObject({}) // true
 *    isObject(null) // false
 */
function isObject(value) {
    return typeof value === "object" && value !== null;
}
exports.isObject = isObject;

/**
 * Returns true if `value` is an Array.
 * @examples
 *    isArray([1, 2, 3])  // true
 *    isArray({ 0: "foo", length: 1 }) // false
 */
var isArray = Array.isArray || function isArray(value) {
  Object.prototype.toString.call(value) === "[object Array]";
}
exports.isArray = isArray;

/**
 * Returns `true` if `value` is an Arguments object.
 * @examples
 *    (function(){ return isArguments(arguments); })(1, 2, 3); // true
 *    isArguments([1,2,3]); // false
 */
function isArguments(value) {
  Object.prototype.toString.call(value) === "[object Arguments]";
}
exports.isArguments = isArguments;

/**
 * Returns true if it is a primitive `value`. (null, undefined, number,
 * boolean, string)
 * @examples
 *    isPrimitive(3) // true
 *    isPrimitive("foo") // true
 *    isPrimitive({ bar: 3 }) // false
 */
function isPrimitive(value) {
  return !isFunction(value) && !isObject(value);
}
exports.isPrimitive = isPrimitive;

/**
 * Returns `true` if given `object` is flat (it is direct decedent of
 * `Object.prototype` or `null`).
 * @examples
 *    isFlat({}) // true
 *    isFlat(new Type()) // false
 */
function isFlat(object) {
  return isObject(object) && (isNull(Object.getPrototypeOf(object)) ||
                              isNull(Object.getPrototypeOf(
                                     Object.getPrototypeOf(object))));
}
exports.isFlat = isFlat;

/**
 * Returns `true` if object contains no values.
 */
function isEmpty(object) {
  if (isObject(object)) {
    for (var key in object)
      return false;
    return true;
  }
  return false;
}
exports.isEmpty = isEmpty;

/**
 * Returns `true` if `value` is an array / flat object containing only atomic
 * values and other flat objects.
 */
function isJSON(value, visited) {
    // Adding value to array of visited values.
    (visited || (visited = [])).push(value);
            // If `value` is an atom return `true` cause it"s valid JSON.
    return  isPrimitive(value) ||
            // If `value` is an array of JSON values that has not been visited
            // yet.
            (isArray(value) &&  value.every(function(element) {
                                  return isJSON(element, visited);
                                })) ||
            // If `value` is a plain object containing properties with a JSON
            // values it"s a valid JSON.
            (isFlat(value) && Object.keys(value).every(function(key) {
                var $ = Object.getOwnPropertyDescriptor(value, key);
                // Check every proprety of a plain object to verify that
                // it"s neither getter nor setter, but a JSON value, that
                // has not been visited yet.
                return  ((!isObject($.value) || !~visited.indexOf($.value)) &&
                        !("get" in $) && !("set" in $) &&
                        isJSON($.value, visited));
            }));
}
exports.isJSON = function (value) {
  return isJSON(value);
};

/**
 * Returns if `value` is an instance of a given `Type`. This is exactly same as
 * `value instanceof Type` with a difference that `Type` can be from a scope
 * that has a different top level object. (Like in case where `Type` is a
 * function from different iframe / jetpack module / sandbox).
 */
function instanceOf(value, Type) {
  var isConstructorNameSame;
  var isConstructorSourceSame;

  // If `instanceof` returned `true` we know result right away.
  var isInstanceOf = value instanceof Type;

  // If `instanceof` returned `false` we do ducktype check since `Type` may be
  // from a different sandbox. If a constructor of the `value` or a constructor
  // of the value"s prototype has same name and source we assume that it"s an
  // instance of the Type.
  if (!isInstanceOf && value) {
    isConstructorNameSame = value.constructor.name === Type.name;
    isConstructorSourceSame = String(value.constructor) == String(Type);
    isInstanceOf = (isConstructorNameSame && isConstructorSourceSame) ||
                    instanceOf(Object.getPrototypeOf(value), Type);
  }
  return isInstanceOf;
}
exports.instanceOf = instanceOf;

/**
 * Function returns textual representation of a value passed to it. Function
 * takes additional `indent` argument that is used for indentation. Also
 * optional `limit` argument may be passed to limit amount of detail returned.
 * @param {Object} value
 * @param {String} [indent="    "]
 * @param {Number} [limit]
 */
function source(value, indent, limit, offset, visited) {
  var result;
  var names;
  var nestingIndex;
  var isCompact = !isUndefined(limit);

  indent = indent || "    ";
  offset = (offset || "");
  result = "";
  visited = visited || [];

  if (isUndefined(value)) {
    result += "undefined";
  }
  else if (isNull(value)) {
    result += "null";
  }
  else if (isString(value)) {
    result += "\"" + value + "\"";
  }
  else if (isFunction(value)) {
    value = String(value).split("\n");
    if (isCompact && value.length > 2) {
      value = value.splice(0, 2);
      value.push("...}");
    }
    result += value.join("\n" + offset);
  }
  else if (isArray(value)) {
    if ((nestingIndex = (visited.indexOf(value) + 1))) {
      result = "#" + nestingIndex + "#";
    }
    else {
      visited.push(value);

      if (isCompact)
        value = value.slice(0, limit);

      result += "[\n";
      result += value.map(function(value) {
        return offset + indent + source(value, indent, limit, offset + indent,
                                        visited);
      }).join(",\n");
      result += isCompact && value.length > limit ?
                ",\n" + offset + "...]" : "\n" + offset + "]";
    }
  }
  else if (isObject(value)) {
    if ((nestingIndex = (visited.indexOf(value) + 1))) {
      result = "#" + nestingIndex + "#"
    }
    else {
      visited.push(value)

      names = Object.keys(value);

      result += "{ // " + value + "\n";
      result += (isCompact ? names.slice(0, limit) : names).map(function(name) {
        var _limit = isCompact ? limit - 1 : limit;
        var descriptor = Object.getOwnPropertyDescriptor(value, name);
        var result = offset + indent + "// ";
        var accessor;
        if (0 <= name.indexOf(" "))
          name = "\"" + name + "\"";

        if (descriptor.writable)
          result += "writable ";
        if (descriptor.configurable)
          result += "configurable ";
        if (descriptor.enumerable)
          result += "enumerable ";

        result += "\n";
        if ("value" in descriptor) {
          result += offset + indent + name + ": ";
          result += source(descriptor.value, indent, _limit, indent + offset,
                           visited);
        }
        else {

          if (descriptor.get) {
            result += offset + indent + "get " + name + " ";
            accessor = source(descriptor.get, indent, _limit, indent + offset,
                              visited);
            result += accessor.substr(accessor.indexOf("{"));
          }

          if (descriptor.set) {
            if (descriptor.get) result += ",\n";
            result += offset + indent + "set " + name + " ";
            accessor = source(descriptor.set, indent, _limit, indent + offset,
                              visited);
            result += accessor.substr(accessor.indexOf("{"));
          }
        }
        return result;
      }).join(",\n");

      if (isCompact) {
        if (names.length > limit && limit > 0) {
          result += ",\n" + offset  + indent + "//...";
        }
      }
      else {
        if (names.length)
          result += ",";

        result += "\n" + offset + indent + "\"__proto__\": ";
        result += source(Object.getPrototypeOf(value), indent, 0,
                         offset + indent);
      }

      result += "\n" + offset + "}";
    }
  }
  else {
    result += String(value);
  }
  return result;
}
exports.source = function (value, indentation, limit) {
  return source(value, indentation, limit);
};
});

require.define("/node_modules/test/logger.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var font = require("ansi-font/index")
var toSource = require("./utils").source

var INDENT = "  "

var report = console.log.bind(console)

function passed(message) {
  return font.green("\u2713 " + message)
}
function failed(message) {
  return font.red("\u2717 " + message)
}
function errored(message) {
  return font.magenta("\u26A1 " + message)
}

function indent(message, indentation) {
  indentation = undefined === indentation ? INDENT : indentation
  message = message || ""
  return message.replace(/^/gm, indentation)
}

function Logger(options) {
  if (!(this instanceof Logger)) return new Logger(options)

  options = options || {}
  var print = options.print || report
  var indentation = options.indentation || ""
  var results = options.results || { passes: [], fails: [], errors: [] }
  this.passes = results.passes
  this.fails = results.fails
  this.errors = results.errors
  results = this


  this.pass = function pass(message) {
    results.passes.push(message)
    print(indent(passed(message), indentation))
  }

  this.fail = function fail(error) {
    results.fails.push(error)
    var message = error.message
    if ("expected" in error)
      message += "\n  Expected: \n" + toSource(error.expected, INDENT)
    if ("actual" in error)
      message += "\n  Actual: \n" + toSource(error.actual, INDENT)
    if ("operator" in error)
      message += "\n  Operator: " + toSource(error.operator, INDENT)
    print(indent(failed(message), indentation))
  }

  this.error = function error(exception) {
    results.errors.push(exception)
    print(indent(errored(exception.stack || exception), indentation))
  }

  this.section = function section(title) {
    print(indent(title, indentation))
    return new Logger({
      print: print,
      indentation: indent(indentation),
      results: results
    })
  }

  this.report = function report() {
    print("Passed:" + results.passes.length +
          " Failed:" + results.fails.length +
          " Errors:" + results.errors.length)
  }
}

Logger.Logger = Logger
module.exports = Logger
});

require.define("/node_modules/test/node_modules/ansi-font/package.json",function(require,module,exports,__dirname,__filename,process){module.exports = {"main":"./index.js"}});

require.define("/node_modules/test/node_modules/ansi-font/index.js",function(require,module,exports,__dirname,__filename,process){/* vim:set ts=2 sw=2 sts=2 expandtab */
/*jshint asi: true newcap: true undef: true es5: true node: true devel: true
         forin: false */
/*global define: true */

(typeof define === "undefined" ? function ($) { $(require, exports, module) } : define)(function (require, exports, module, undefined) {

"use strict";

var ESC = '\u001b['

// http://en.wikipedia.org/wiki/ANSI_escape_code#graphics
var SGR_STYLES = {
  bold:           [ 1,  22 ],
  italic:         [ 3,  23 ],
  underline:      [ 4,  24 ],
  blink:          [ 5,  25 ],

  inverse:        [ 7,  27 ],

  frame:         [ 51, 54 ],
  encircle:      [ 52, 54 ],

  overline:       [ 53, 55 ],
  strikethrough:  [ 53, 55 ]
}
var SGR_COLORS = {}
var SGR_BACKROUNDS = {}
var COLORS = [
  'black',
  'red',
  'green',
  'yellow',
  'blue',
  'magenta',
  'cyan',
  'white'
]

COLORS.forEach(function(color, index) {
  SGR_COLORS[color] = [ 30 + index, 39 ]
  SGR_BACKROUNDS[color] = [ 40 + index, 49 ]
})

function sgr(options, id, message) {
  var params = options[id]
  if (params) message = ESC + params[0] + 'm' + message + ESC + params[1] + 'm'
  return message
}

exports.style = sgr.bind(null, SGR_STYLES)
exports.color = sgr.bind(null, SGR_COLORS)
exports.background = sgr.bind(null, SGR_BACKROUNDS)

Object.keys(SGR_STYLES).forEach(function(name) {
  exports[name] = exports.style.bind(null, name)
})
Object.keys(SGR_COLORS).forEach(function(name) {
  exports[name] = exports.color.bind(null, name)
  exports['bg' + name] = exports.background.bind(null, name)
})

var index = 0
while(index++ < 256) {
  SGR_COLORS[index] = ['38;5;' + index, 39]
  SGR_BACKROUNDS[index] = ['48;5;' + index, 39]
}

});
});

require.define("/test/map.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var test = require("./util/test")
var delay = require("../delay")
var concat = require("../concat")
var capture = require("../capture")
var map = require("../map")
var into = require("../into")

exports["test map sequence"] = function(assert) {
  var called = 0
  var source = [ 1, 2, 3 ]
  var actual = map(source, function(item) {
    called = called + 1
    return item + 10
  })

  assert.equal(called, 0, "map does not invokes until result is reduced")
  assert.deepEqual(into(actual), [ 11, 12, 13 ], "values are mapped")
  assert.equal(called, 3, "mapper called once per item")
}

exports["test map value"] = function(assert) {
  var called = 0
  var actual = map(7, function(item) {
    called = called + 1
    return item + 10
  })

  assert.equal(called, 0, "map does not invokes until result is reduced")
  assert.deepEqual(into(actual), [ 17 ], "values is mapped")
  assert.equal(called, 1, "mapper called once per item")
}

exports["test map empty"] = test(function(assert) {
  var actual = map([], function(element) {
    throw Error("map fn was executed")
  })

  assert(actual, [], "mapping empty is empty")
})


exports["test number map"] = test(function(assert) {
  var numbers = [1, 2, 3, 4]
  var actual = map(numbers, function(number) { return number * 2 })

  assert(actual, [2, 4, 6, 8], "numbers are doubled")
})

exports["test map with async stream"] = test(function(assert) {
  var source = delay([5, 4, 3, 2, 1])
  var actual = map(source, function(x) { return x + 1 })
  assert(actual, [6, 5, 4, 3, 2], "async number stream is incermented")
})

exports["test map broken stream"] = test(function(assert) {
  var boom = Error("Boom!")
  var source = concat([3, 2, 1], boom)
  var mapped = map(delay(source), function(x) { return x * x })
  var actual = capture(mapped, function(error) { return error.message })

  assert(actual, [9, 4, 1, boom.message], "errors propagate")
})

if (require.main === module)
  require("test").run(exports)
});

require.define("/test/take.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var test = require("./util/test")
var concat = require("../concat")
var delay = require("../delay")
var capture = require("../capture")

var into = require("../into")
var take = require("../take")


exports["test take"] = function(assert) {
  var actual = take([ 1, 2, 3, 4 ], 2)

  assert.deepEqual(into(actual), [ 1, 2 ], "picked two items")
  assert.deepEqual(into(actual), [ 1, 2 ], "can be re-reduced same")
}

exports["test take none"] = function(assert) {
  var actual = take([ 1, 2, 3, 4 ], 0)

  assert.deepEqual(into(actual), [], "picks none on 0")
}

exports["test take all"] = function(assert) {
  var actual = take([ 1, 2, 3, 4 ], 100)

  assert.deepEqual(into(actual), [ 1, 2, 3, 4 ],
                   "picks all if has less than requested")
}

exports["test take empty"] = test(function(assert) {
  var actual = take([], 100)

  assert(actual, [], "nothing to take from empy")
})

exports["test take more than have"] = test(function(assert) {
  var actual = take([1, 2, 3], 5)

  assert(actual, [1, 2, 3], "can't take more then contains")
})

exports["test take falls back to all"] = test(function(assert) {
  var actual = take([1, 2, 3], Infinity)

  assert(actual, [1, 2, 3], "taking infinity takes as much as it has")
})

exports["test take may be given 0"] = test(function(assert) {
  var actual = take([1, 2, 3], 0)

  assert(actual, [], "taking zero returns equivalent")
})

exports["test take on async stream"] = test(function(assert) {
  var delayed = delay([5, 4, 3, 2, 1])
  var actual = take(delayed, 3)

  assert(actual, [5, 4, 3], "works with async streams")
})

exports["test take before error"] = test(function(assert) {
  var boom = Error("Boom!")
  var delayed = delay(concat([3, 2, 1], boom))
  var recovered = capture(delayed, function(e) { return e.message })
  var actual = take(recovered, 3)

  assert(actual, [3, 2, 1], "errors that are not reached are ignored")
})

exports["test error propagation"] = test(function(assert) {
  var boom = Error("Boom!")
  var delayed = delay(concat([3, 2, 1], boom))
  var recovered = capture(delayed, function(e) { return e.message })
  var actual = take(recovered, 5)

  assert(actual, [3, 2, 1, boom.message], "errors with-in take range propagate")
})

if (module == require.main)
  require("test").run(exports)
});

require.define("/take.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var reducible = require("./reducible")
var accumulate = require("./accumulate")
var end = require("./end")

function take(source, n) {
  /**
  Returns sequence of first `n` items of the given `source`. If `source`
  contains less items than `n` then that's how  much items return sequence
  will contain.

  ## Example

  print(take([ 1, 2, 3, 4, 5 ], 2))   // => < 1 2 >
  print(take([ 1, 2, 3 ], 5))         // => < 1 2 3 >
  **/

  // If take `0` then optimize by returning an empty if less then `0`
  // then just return `source` back.
  if (n === 0) return void(0)
  if (n < 0) return source
  return reducible(function accumulateTake(next, initial) {
    // Capture `n` into count, since modifying `n` directly will have side
    // effects on subsequent calls.
    var count = n
    accumulate(source, function accumulateTakeSource(value, result) {
      count = count - 1
      result = next(value, result)

      // If we have not taken `n` items yet just pass result back. Otherwise
      // pass `end` of stream to a consumer. Note `reducible` will return
      // `reduced(result)` back signaling source it should stop.
      return count > 0 ? result :
             next(end)
    }, initial)
  })
}

module.exports = take
});

require.define("/test/take-while.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var test = require("./util/test")
var lazy = require("./util/lazy")
var concat = require("../concat")
var delay = require("../delay")
var capture = require("../capture")

var takeWhile = require("../take-while")
var into = require("../into")

exports["test takeWhile"] = function(assert) {
  var called = 0
  var source = [ 1, 2, 3, 4 ]
  var actual = takeWhile(source, function(item) {
    called = called + 1
    return item <= 2
  })

  assert.equal(called, 0, "takeWhile does not invokes until result is reduced")
  assert.deepEqual(into(actual), [ 1, 2 ], "items were taken")
  assert.equal(called, 3, "taker called until it returns false")
  assert.deepEqual(into(actual), [ 1, 2 ], "can be re-reduced")
}

exports["test takeWhile none"] = function(assert) {
  var called = 0
  var source = [ 1, 2, 3, 4 ]
  var actual = takeWhile(source, function(item) {
    called = called + 1
    return false
  })

  assert.equal(called, 0, "takeWhile does not invokes until result is reduced")
  assert.deepEqual(into(actual), [], "0 items were taken")
  assert.equal(called, 1, "taker called once")
}

exports["test takeWhile all"] = function(assert) {
  var called = 0
  var source = [ 1, 2, 3 ]
  var actual = takeWhile(source, function(item) {
    called = called + 1
    return true
  })

  assert.equal(called, 0, "takeWhile does not invokes until result is reduced")
  assert.deepEqual(into(actual), [ 1, 2, 3 ], "all items were taken")
  assert.equal(called, 3, "taker called on each item")
}

exports["test take while on empty"] = test(function(assert) {
  var called = 0
  var source = takeWhile([], function(n) {
    called = called + 1
    return n <= 9
  })

  var actual = concat(source,
                      "x",
                      lazy(function() { return called }))

  assert(actual, ["x", 0], "nothing to take from empty no calls to f")
})

exports["test take while"] = test(function(assert) {
  var called = 0
  var source = takeWhile([1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12], function(n) {
    called = called + 1
    return n <= 9
  })

  var actual = concat(source,
                      "x",
                      lazy(function() { return called }))


  assert(actual, [1, 2, 3, 4, 5, 6, 7, 8, 9, "x", 10],
         "called until returned false")
})

exports["test take while ends"] = test(function(assert) {
  var called = 0
  var source = takeWhile([1, 2, 3, 4, 5, 6, 7], function(n) {
    called = called + 1
    return n <= 9
  })

  var actual = concat(source,
                      "x",
                      lazy(function() { return called }))


  assert(actual, [1, 2, 3, 4, 5, 6, 7, "x", 7],
         "takes until exhasted")
})

exports["test take while on async stream"] = test(function(assert) {
  var called = 0
  var source =  delay([1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12])
  var taken = takeWhile(source, function(n) {
    called = called + 1
    return n <= 9
  })
  var actual = concat(taken,
                      "x",
                      lazy(function() { return called }))



  assert(actual, [1, 2, 3, 4, 5, 6, 7, 8, 9, "x", 10],
        "called until returned false")
})

exports["test error propagation in to take while"] = test(function(assert) {
  var called = 0, boom = Error("boom!")
  var source = concat([1, 2, 3, 4, 5], boom, [6, 7])
  var taken = takeWhile(source, function(n) {
    called = called + 1
    return n <= 9
  })
  var recovered = capture(taken, function(error) {
    return error.message
  })
  var actual = concat(recovered,
                      "x",
                      lazy(function() { return called }))

  assert(actual, [1, 2, 3, 4, 5, boom.message, "x", 5], "called until error")
})


if (module == require.main)
  require("test").run(exports)
});

require.define("/take-while.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var reducer = require("./reducer")
var end = require("./end")


var takeWhile = reducer(function takeWhile(predicate, next, value, result) {
  /**
  Returns a sequence of successive items from `source` while `predicate(item)`
  returns `true`. `predicate` must be free of side-effects.

  ## Example

  var digits = takeWhile([ 2, 7, 10, 23 ], function(x) { return x < 10 })
  print(digits)   // => < 2 7 >
  **/
  return predicate(value) ? next(value, result) :
         // Predicate returned `false` just pass `end`. Reducer will take
         // care of returning `reduced(result)` back.
         next(end)
})

module.exports = takeWhile
});

require.define("/test/drop.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var test = require("./util/test")
var lazy = require("./util/lazy")

var into = require("../into")
var drop = require("../drop")
var delay = require("../delay")
var concat = require("../concat")
var capture = require("../capture")


exports["test drop"] = function(assert) {
  var actual = drop([ 1, 2, 3, 4 ], 2)

  assert.deepEqual(into(actual), [ 3, 4 ], "skipped two items")
  assert.deepEqual(into(actual), [ 3, 4 ], "can be re-reduced same")
}

exports["test drop none"] = function(assert) {
  var actual = drop([ 1, 2, 3, 4 ], 0)

  assert.deepEqual(into(actual), [ 1, 2, 3, 4 ], "skips none on 0")
}

exports["test drop all"] = function(assert) {
  var actual = drop([ 1, 2, 3, 4 ], 100)

  assert.deepEqual(into(actual), [],
                   "skips all if has less than requested")
}

exports["test drop empty"] = test(function(assert) {
  var actual = drop([], 100)

  assert(actual, [], "drop on empty is empty")
})



exports["test drop on sync stream"] = test(function(assert) {
  var actual = drop([1, 2, 3, 4], 3)

  assert(actual, [4], "dropped all except last")
})

exports["test drop falls back to 1"] = test(function(assert) {
  var actual = drop([1, 2, 3, 4], Infinity)

  assert(actual, [], "dropped every item")
})

exports["test drop can take 0"] = test(function(assert) {
  var actual = drop([1, 2, 3, 4], 0)

  assert(actual, [1, 2, 3, 4], "dropped none")
})

exports["test drop more than have"] = test(function(assert) {
  var actual = drop([1, 2, 3, 4], 5)

  assert(actual, [], "dropped more the contained")
})

exports["test drop of async stream"] = test(function(assert) {
  var actual = drop(delay([5, 4, 3, 2, 1]), 2)

  assert(actual, [3, 2, 1], "drop 2 from async")
})

exports["test drop on stream with error"] = test(function(assert) {
  var boom = Error("Boom!")
  var dropped = drop(delay(concat([4, 3, 2, 1], boom)), 2)
  var actual = capture(dropped, function(error) { return error.message })

  assert(actual, [2, 1, boom.message], "dropped on broken stream")
})

exports["test drop on stream with error in head"] = test(function(assert) {
  var boom = Error("Boom!")
  var dropped = drop(delay(concat(boom, [4, 3, 2, 1])), 2)
  var actual = capture(dropped, function(error) { return error.message })

  assert(actual, [boom.message], "attempt to drop on early errors")
})


if (require.main === module)
  require("test").run(exports)
});

require.define("/drop.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var reducible = require("./reducible")
var accumulate = require("./accumulate")
var isError = require("./is-error")
var end = require("./end")

function drop(source, n) {
  /**
  Returns sequence of all `source`'s items after `n`-th one. If source contains
  less then `n` items empty sequence is returned.

  ## Example

  print(drop([ 1, 2, 3, 4 ], 2))  // => <stream 3 4 />
  print(drop([ 1, 2, 3 ], 5))     // => <stream />
  **/

  // If drop `<= 0` then optimize by returning source itself. If `Infinity`
  // return empty.
  if (n <= 0) return source
  if (n === Infinity) return void(0)
  return reducible(function accumulateDrop(next, initial) {
    var count = n
    accumulate(source, function accumulateDropSource(value, result) {
      // If value is end of collection or is an error (which also includes
      // end of collection) just pass it through, `reducible` will take care
      // of everything.
      if (value === end) return next(value, result)
      if (isError(value)) return next(value, result)
      // If count of items has reached `0` just keep on passing values.
      if (count === 0) return next(value, result)
      // Otherwise just decrement count and return `result`.
      count = count - 1
      return result
    }, initial)
  })
}

module.exports = drop
});

require.define("/test/drop-while.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var test = require("./util/test")
var lazy = require("./util/lazy")

var concat = require("../concat")
var dropWhile = require("../drop-while")
var into = require("../into")
var delay = require("../delay")
var capture = require("../capture")


exports["test drop while"] = function(assert) {
  var called = 0
  var source = [ 1, 2, 3, 4 ]
  var actual = dropWhile(source, function(item) {
    called = called + 1
    return item <= 2
  })

  assert.equal(called, 0, "dropWhile does not invokes until result is reduced")
  assert.deepEqual(into(actual), [ 3, 4 ], "items were dropped")
  assert.equal(called, 3, "dropWhile called until it returns false")
  assert.deepEqual(into(actual), [ 3, 4 ], "can be re-reduced")
}

exports["test drop while none"] = function(assert) {
  var called = 0
  var source = [ 1, 2, 3 ]
  var actual = dropWhile(source, function(item) {
    called = called + 1
    return false
  })

  assert.equal(called, 0, "dropWhile does not invokes until result is reduced")
  assert.deepEqual(into(actual), [ 1, 2, 3 ], "0 items were dropped")
  assert.equal(called, 1, "dropper called only once")
}

exports["test drop while all"] = function(assert) {
  var called = 0
  var source = [ 1, 2, 3 ]
  var actual = dropWhile(source, function(item) {
    called = called + 1
    return true
  })

  assert.equal(called, 0, "dropWhile does not invokes until result is reduced")
  assert.deepEqual(into(actual), [], "all items were dropped")
  assert.equal(called, 3, "dropper called on each item")
}

exports["test drop while on empty"] = test(function(assert) {
  var called = 0
  var dropped = dropWhile([], function(n) {
    called = called + 1
    return n <= 9
  })

  var actual = concat(dropped,
                      lazy(function() { return called }))

  assert(actual, [0], "nothing to drop")
})

exports["test drop.while"] = test(function(assert) {
  var called = 0
  var dropped = dropWhile([1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12], function(n) {
    called = called + 1
    return n <= 9
  })
  var actual = concat(dropped,
                      lazy(function() { return called }))

  assert(actual, [10, 11, 12, 10], "called until returned false")
})

exports["test drop.while end"] = test(function(assert) {
  var called = 0
  var dropped = dropWhile([1, 2, 3, 4, 5, 6, 7], function(n) {
    called = called + 1
    return n <= 9
  })
  var actual = concat(dropped,
                      lazy(function() { return called }))

  assert(actual, [7], "called until returned false")
})

exports["test drop while on async stream"] = test(function(assert) {
  var called = 0
  var stream = delay([1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12])
  var dropped = dropWhile(stream, function(n) {
    called = called + 1
    return n <= 9
  })
  var actual = concat(dropped,
                      lazy(function() { return called }))

  assert(actual, [10, 11, 12, 10], "called until returned false")
})

exports["test error propagation in drop.while"] = test(function(assert) {
  var called = 0, boom = Error("boom!")
  var stream = concat([1, 2, 3, 4, 5], boom, [6, 7])
  var dropped = dropWhile(stream, function(n) {
    called = called + 1
    return n <= 3
  })
  var captured = capture(dropped, function(error) { return error.message })
  var actual = concat(captured,
                      lazy(function() { return called }))

  assert(actual, [4, 5, boom.message, 4], "called until error ocurred")
})

if (require.main === module)
  require("test").run(exports)
});

require.define("/drop-while.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var reducible = require("./reducible")
var accumulate = require("./accumulate")
var isError = require("./is-error")
var end = require("./end")

function dropWhile(source, predicate) {
  /**
  Returns a sequence of the items in `source` starting from the first
  item for which `predicate(item)` returns `false`.

  ## Example

  var numbers = dropWhile([ 2, 7, 10, 23 ], function(value) {
    return value < 10
  })
  print(numbers)   // => < 10 23 >
  **/
  return reducible(function accumulateDropWhile(next, initial) {
    var dropped = false
    accumulate(source, function accumulateDropWhileSource(value, result) {
      // If value is end of collection or is an error (which also includes
      // end of collection) just pass it through, `reducible` will take care
      // of everything.
      if (value === end) return next(value, result)
      if (isError(value)) return next(value, result)

      // If already dropped all the intended items (if `dropped` is already
      // being set to `true` or if current predicate returns `false`). Then
      // just keep on passing values.
      if (dropped || (dropped = !predicate(value))) return next(value, result)

      return result
    }, initial)
  })
}

module.exports = dropWhile
});

require.define("/test/into.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var into = require("../into")

exports["test into"] = function(assert) {
  var source = [ 1, 2, 3 ]
  assert.deepEqual(into(source), [ 1, 2, 3 ],
                   "returns identical")
  assert.notEqual(into(source), source,
                  "but different one")
}

exports["test into buffer"] = function(assert) {
  var buffer = [ 0 ]
  assert.equal(into([ 1, 2, 3 ], buffer), buffer,
               "reduces into buffer if provided")
  assert.deepEqual(buffer, [ 0, 1, 2, 3 ],
                   "pre-existing items in buffer are kept")
}

if (module == require.main)
  require("test").run(exports)
});

require.define("/test/concat.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var test = require("./util/test")
var concat = require("../concat")
var into = require("../into")
var delay = require("../delay")
var capture = require("../capture")

exports["test concat"] = function(assert) {
  var actual = concat([ 1 ], [ 2, 3 ], [ 4, 5, 6 ])

  assert.deepEqual(into(actual),
                   [ 1, 2, 3, 4, 5, 6 ],
                   "reducers were joined")
  assert.deepEqual(into(actual),
                   [ 1, 2, 3, 4, 5, 6 ],
                   "can be re-reduced")
}

exports["test concat empty streams"] = function(assert) {
  var actual = concat([], [])

  assert.deepEqual(into(actual), [], "concat on empty sequences is empty")
}

exports["test concat empty"] = function(assert) {
  var actual = concat([1, 2], [])

  assert.deepEqual(into(actual), [ 1, 2 ], "concat with empty returns same")
}

exports["test concat to empty"] = function(assert) {
  var actual = concat([], [3, 4])

  assert.deepEqual(into(actual), [ 3, 4 ], "concat to empty returns same")
}

exports["test concat many streams"] = function(assert) {
  var actual = concat([1, 2], [], ["a", "b"], [])

  assert.deepEqual(into(actual), [ 1, 2, "a", "b" ], "many concats work")
}

exports["test concat sync & async streams"] = test(function(assert) {
  var async = delay([3, 2, 1])
  var actual = concat(concat(async, []),
                      async,
                      ["a", "b"])

  assert(actual, [3, 2, 1, 3, 2, 1, "a", "b"], "concat sync async works")
})

exports["test concat & reconcat"] = test(function(assert) {
  var async = delay([2, 1])
  var stream = concat(async, ["a", "b"])
  var actual = concat(stream, ["||"], stream)

  assert(actual, [2, 1, "a", "b", "||", 2, 1, "a", "b"], "concat reconcat")
})

exports["test map broken stream"] = test(function(assert) {
  var boom = Error("Boom!")
  var async = delay(concat([3, 2, 1], boom))
  var concated = concat([">"], async, [1, 2], async)
  var actual = capture(concated, function(error) { return error.message })

  assert(actual, [">", 3, 2, 1, boom.message], "test concat on broken stream")
})

exports["test concat error and sequence"] = test(function(assert) {
  var boom = Error("Boom!")
  var concated = concat(boom, [ 1, 2, 3 ])
  var actual = capture(concated, function(error) {
    return error.message
  })


  assert(actual, [boom.message], "sequence has an error")
})

exports["test concat sequence & error"] = test(function(assert) {
  var boom = Error("Boom!")
  var concated = concat([ 1, 2, 3 ], boom)
  var actual = capture(concated, function(error) {
    return error.message
  })


  assert(actual, [1, 2, 3, boom.message], "sequence has an error")
})

exports["test concat sequence & error & sequence"] = test(function(assert) {
  var boom = Error("Boom!")
  var concated = concat([ 1, 2, 3 ], boom, [ 4, 5, 6 ])
  var actual = capture(concated, function(error) {
    return error.message
  })


  assert(actual, [1, 2, 3, boom.message], "sequence has an error")
})

if (module == require.main)
  require("test").run(exports)
});

require.define("/test/flatten.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var test = require("./util/test")
var lazy = require("./util/lazy")

var concat = require("../concat")
var into = require("../into")
var delay = require("../delay")
var capture = require("../capture")
var map = require("../map")
var expand = require("../expand")
var take = require("../take")
var flatten = require("../flatten")
var into = require("../into")
var sequential = require("../sequential")

exports["test flatten"] = function(assert) {
  var source = [ [ 1 ], [ 2, 3 ], [ 4, 5, 6 ] ]
  var actual = flatten(source)

  assert.deepEqual(into(actual),
                   [ 1, 2, 3, 4, 5, 6 ],
                   "flatten reducers")
  assert.deepEqual(into(actual),
                   [ 1, 2, 3, 4, 5, 6 ],
                   "can be re-reduced")
  assert.deepEqual(source, [ [ 1 ], [ 2, 3 ], [ 4, 5, 6 ] ],
                   "no changes to a source")
}

exports["test flatten stream of empty streams"] = test(function(assert) {
  var actual = flatten([[], []])

  assert(actual, [], "flatten of empty streams is empty")
})

exports["test flatten empty & non-empty"] = test(function(assert) {
  var actual = flatten([[], [1, 2], []])

  assert(actual, [1, 2], "flatten with empties is non-empty")
})

exports["test flatten flattened"] = test(function(assert) {
  var stream = flatten([[1, 2], ["a", "b"]])
  var actual = flatten([[">"], stream, []])

  assert(actual, [">", 1, 2, "a", "b"], "flattening flattened works")
})

exports["test flatten sync & async streams"] = test(function(assert) {
  var async = delay([3, 2, 1])
  var actual = flatten([async, ["|"], async, ["a", "b"], []])

  assert(actual, ["|", "a", "b", 3, 3, 2, 2, 1, 1], "orders by time")
})

exports["test flatten sequential sync & async streams"] = test(function(assert) {
  var async = delay([3, 2, 1])
  var actual = flatten(sequential([async, ["|"], async, ["a", "b"], []]))

  assert(actual, [3, 2, 1, "|", 3, 2, 1, "a", "b"],
         "sequentialized flatten")
})

exports["test flatten with broken stream"] = test(function(assert) {
  var boom = Error("Boom!")
  var async = delay(concat([3, 2, 1], boom))
  var flattened = flatten([[">"], async, [1, 2]])
  var actual = capture(flattened, function(error) {
    return error.message
  })

  assert(actual, [">", 1, 2, 3, 2, 1, boom.message], "errors propagate")
})

exports["test flatten with broken sequential stream"] = test(function(assert) {
  var boom = Error("Boom!")
  var async = delay(concat([3, 2, 1], boom))
  var flattened = flatten(sequential([[">"], async, [1, 2]]))
  var actual = capture(flattened, function(error) {
    return error.message
  })

  assert(actual, [">", 3, 2, 1, boom.message], "errors propagate")
})

exports["test flatten sequential async stream of streams"] = test(function(assert) {
  var async = delay([3, 2, 1])
  var actual = flatten(sequential([[], [1, 2], async, ["a", "b"], async]))

  assert(actual, [1, 2, 3, 2, 1, "a", "b", 3, 2, 1], "mixed stream works")
})

exports["test flatten async stream of streams"] = test(function(assert) {
  var async = delay([3, 2, 1])
  var actual = flatten([[], [1, 2], async, ["a", "b"], async])

  assert(actual, [1, 2, "a", "b", 3, 3, 2, 2, 1, 1], "mixed stream works")
})

exports["test flatten async sequential stream of streams"] = test(function(assert) {
  var async = delay([3, 2, 1])
  var actual = flatten(sequential([[], [1, 2], async, ["a", "b"], async]))

  assert(actual, [1, 2, 3, 2, 1, "a", "b", 3, 2, 1],
         "sequential stream of streams stream works")
})

if (module == require.main)
  require("test").run(exports)
});

require.define("/expand.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var flatten = require("./flatten")
var map = require("./map")

function expand(source, f) {
  /**
  Takes `source` sequence maps each item via `f` to a new sequence
  and then flattens them down into single form sequence. Note that
  returned sequence will have items ordered by time and not by index,
  if you wish opposite you need to force sequential order by wrapping
  `source` into `sequential` before passing it.

  ## Example

  var sequence = expand([ 1, 2, 3 ], function(x) {
    return [ x, x * x ]
  })
  print(sequence)   // => <stream 1 1 2 4 3 9 />

  **/
  return flatten(map(source, f))
}

module.exports = expand

});

require.define("/flatten.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var end = require("./end")
var reduce = require("./reduce")
var accumulate = require("./accumulate")
var reducible = require("./reducible")
var eventual = require("eventual/decorate")
var when = require("eventual/when")

var first = eventual(function(value) { return value })

function flatten(source) {
  /**
  Flattens given sequence of sequences to a sequence with items of
  all nested sequences. Note that items in the resulting sequence
  are ordered by the time rather then index, in other words if
  item from the second sub-sequence is deliver earlier then the item
  from first sub-sequence it will in appear earlier in the resulting
  sequence too. If you need to keep order by index you should enforce
  sequential ordering by wrapping `source` into `sequential` before
  passing it to flatten.

  print(flatten([ [1, 2], [ 3, 4 ] ]))  // => < 1 2 3 4 />
  **/
  return reducible(function(next, initial) {
    // Result is accumulated into this outer variable as items from then
    // nested collections can come in time based order so it won't be passed
    // to in to reducer function.
    var result = initial
    var promise = reduce(source, function(promise, nested) {
      // we group results to make sure flattened stream won't finish until
      // all the streams are finished.
      return first(reduce(nested, function(_, value) {
        result = next(value, result)
        return result
      }), promise)
    }, source)
    when(promise, function(value) { next(end, value) }, next)
  })
}

module.exports = flatten
});

require.define("/node_modules/eventual/decorate.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var apply = require("./apply")

// Define a shortcut for `Array.prototype.slice.call`.
var unbind = Function.call.bind(Function.bind, Function.call)
var slice = Array.slice || unbind(Array.prototype.slice)

// Decorator function composes function which can take eventual values as
// arguments, in which case it returns eventual result that is delivered
// result of applying results of eventuals to a decorated `f`. If result is
// delivered in sync delivery value is returned instead.
module.exports = function eventual(f) {
  return function eventually() {
    return apply(f, slice(arguments))
  }
}
});

require.define("/node_modules/eventual/apply.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var when = require("./when")
var group = require("./group")

// Function can be use with eventual values, it treats each of it's arguments
// as eventual value and returns a fresh one in return. Once all of the
// arguments are delivered first one is invoked with rest and return value
// is delivered to the resulting eventual. If everything happens synchronously
// actual value is returned.
module.exports = function invoke(f, params) {
  var eventuals = params.slice(0)
  eventuals.unshift(f)
  return when(group(eventuals), function(values) {
    var f = values.shift()
    return f.apply(f, values)
  })
}
});

require.define("/node_modules/eventual/group.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var when = require("./when")
var unbind = Function.call.bind(Function.bind, Function.call)
var slice = Array.slice || unbind(Array.prototype.slice)

// Utility function takes array of eventual values and return single
// eventual value that is delivered an array of delivery values for
// those eventuals. If any of the eventuals is delivered error that
// error is delivered instead.
module.exports = function group(eventuals) {
  return slice(eventuals).reduce(function(eventuals, eventual) {
    return when(eventual, function(value) {
      return when(eventuals, function(values) {
        values.push(value)
        return values
      })
    })
  }, [])
}
});

require.define("/sequential.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var reducible = require("./reducible")
var accumulate = require("./accumulate")
var reduce = require("./reduce")
var eventual = require("eventual/decorate")
var when = require("eventual/when")
var end = require("./end")

function sequential(source) {
  /**
  Takes `source` sequence and returns decorated version which intercepts
  into reduction such that any transformation on the result will have
  guaranteed sequential order. This decorator is useful with functions
  like `flatten` and `expand` when order of items in resulting sequences
  is desired to be index based rather then time based. Note that this
  will buffer items that are delivered earlier then their order so keep
  that in mind and use it with extra care.
  **/
  return reducible(function reduceReducible(next, initial) {
    var result = reduce(source, eventual(function(value, result) {
      return next(result, value)
    }), initial)
    when(result, function(result) { next(end, result) }, next)
  })
}

module.exports = sequential
});

require.define("/test/zip.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var zip = require("../zip")
var into = require("../into")
var concat = require("../concat")
var capture = require("../capture")

exports["test zip with empty"] = function(assert) {
  var actual = into(zip([], [1, 2]))

  assert.deepEqual([], actual, "empty sequence zips to empty sequence")
}

exports["test zip 2 sequences"] = function(assert) {
  var actual = zip([ 1, 2, 3, 4 ], ["a", "b", "c", "d"])

  assert.deepEqual(into(actual), [
    [ 1, "a" ], [ 2, "b" ], [ 3, "c" ], [ 4, "d" ]
  ], "zip returns pairs of items")
}

exports["test zip sync stream with async stream"] = function(assert) {
  var actual = zip([5, 4, 3, 2, 1],
                   ["a", "b", "c", "d", "e"],
                   ["~", "@", "!", "#"])

  assert.deepEqual(into(actual), [
    [ 5, "a", "~"  ],
    [ 4, "b", "@" ],
    [ 3, "c", "!" ],
    [ 2, "d", "#" ]
  ], "zip three sequences into one")
}

exports["test zip with late error"] = function(assert) {
  var boom = Error("boom")
  var actual = zip(concat([3, 2, 1], boom), ["a", "b", "c"])

  assert.deepEqual(into(actual), [
    [ 3, "a" ],
    [ 2, "b" ],
    [ 1, "c" ]
  ], "ziping is finished before error occured")
}

exports["test zip with early error"] = function(assert) {
  var boom = Error("Boom!!")
  var zipped = zip(concat([1, 2, 3], boom), ["a", "b", "c", "d"])
  var actual = capture(zipped, function(error) { return [ "error" ] })

  assert.deepEqual(into(actual), [
    [ 1, "a" ],
    [ 2, "b" ],
    [ 3, "c" ],
    "error"
  ], "early erros propagate through the zipping")
}

if (module == require.main)
  require("test").run(exports)
});

require.define("/zip.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var accumulate = require("./accumulate")
var reduced = require("./reduced")
var isReduced = require("./is-reduced")
var map = require("./map")
var isError = require("./is-error")
var end = require("./end")

var slicer = Array.prototype.slice

function makeAccumulator(side) {
  var other = side === "left" ? "right" : "left"
  return function accumulate(value, state) {
    var queue = state[side]
    var buffer = state[other]
    var dispatch = state.next
    // If consumer finished consumption, then notify stream.
    if (state.closed)  return state.result
    // If this is an end of this stream, close a queue to indicate
    // no other value will be queued.
    else if (value === end) {
      if (isReduced(state)) return state
      queue.closed = true
      // If queue is empty, dispatch end of stream.
      if (!queue.length) {
        dispatch(value, state.result)
        state.left = state.right = state.next = null
        state.closed = true
        state.result = reduced(result)
      }
    }
    else {
      queue.push(value)
      // If there is a buffered value on both streams shift and dispatch.
      if (buffer.length) {
        if (isError(buffer[0]))
          dispatch(buffer.shift(), state.result)
        else if (isError(queue[0]))
          dispatch(queue.shift(), state.result)

        if (buffer.length && queue.length) {
          var result = dispatch([
            state.left.shift(),
            state.right.shift()
          ], state.result)
          // If consumer is done consumption or if buffer is empty and closed
          // dispatch end, and mark stream ended to stop streams and queueing
          // values too.
          if (isReduced(result) || (buffer.closed && !buffer.length)) {
            // Dispatch end of stream and cleanup state attributes.
            dispatch(end, result)
            state.left = state.right = state.next = null
            state.closed = true
            state.result = reduced(result)
          } else {
            state.result = result
          }
        }
      }
    }
    return state
  }
}

var accumulateLeft = makeAccumulator("left")
var accumulateRight = makeAccumulator("right")

function Zip() {}
accumulate.define(Zip, function(zipped, next, start) {
  var state = { result: start, next: next, left: [], right: [] }
  accumulate(zipped.left, accumulateLeft, state)
  accumulate(zipped.right, accumulateRight, state)
})

function array(item) { return [item] }

function unite(value) {
  value[0].push(value[1])
  return value[0]
}

function concatzip(zipped, sequence) {
  return map(zip(zipped, sequence), unite)
}

function zip(left, right/*, ...rest*/) {
  switch (arguments.length) {
    case 1:
      return map(left, array)
    case 2:
      var value = new Zip()
      value.left = left
      value.right = right
      value.leftQueue = []
      value.rightQueue = []
      return value
    default:
      return slicer.call(arguments, 2).reduce(concatzip, zip(left, right))
  }
}

module.exports = zip
});

require.define("/test/capture.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var test = require("./util/test")
var concat = require("../concat")
var into = require("../into")
var delay = require("../delay")
var capture = require("../capture")
var map = require("../map")
var expand = require("../expand")
var take = require("../take")
var lazy = require("./util/lazy")

exports["test capture empty stream"] = test(function(assert) {
  var called = 0
  var captured = capture([], function final() {
    called = called + 1
  })
  var actual = concat(captured, lazy(function() { return called }))


  assert(actual, [0], "error handler was not called")
})

exports["test capture with a non-empty stream"] = test(function(assert) {
  var actual = capture([1, 2], function() { return [3, 4] })

  assert(actual, [1, 2], "error handler has is not called")
})

exports["test error recovery"] = test(function(assert) {
  var boom = Error("Boom!")
  var actual = capture(boom, function catcher(e) {
    return [ "catch", e.message ]
  })

  assert(actual, ["catch", boom.message ], "error handler is called")
})

exports["test errors can be ignored"] = test(function(assert) {
  var boom = Error("Boom!")
  var brax = Error("brax")
  var source = concat(["h", "i"], boom)
  var captured = capture(source, function cacher(e) {
    return [e.message, brax]
  })
  var actual = capture(captured, function(e) {
    return e.message
  })


  assert(actual, ["h", "i", boom.message, brax.message],
                  "recovery code still may leak errors")
})


exports["test capture error every time"] = test(function(assert) {
  var boom = Error("Boom!!")
  var calls = 0
  var reason
  var captured = capture(boom, function(error) {
    calls = calls + 1
    reason = error
    return
  })

  var actual = concat(captured,
                      lazy(function() { return calls }),
                      lazy(function() { return reason.message }),
                      captured,
                      lazy(function() { return calls }),
                      lazy(function() { return reason.message }))


  assert(actual, [1, boom.message, 2, boom.message],
         "error handler is called each time")
})

exports["test substitution is lazy"] = test(function(assert) {
  var calls = 0
  var boom = Error("boom")
  var captured = capture([1, 2, 3, 4, boom], function(error) {
    calls = calls + 1
    return [5, 6, 7]
  })

  var actual = concat(take(captured, 1),
                      lazy(function() { return calls }),
                      captured,
                      lazy(function() { return calls }))

  assert(actual, [
    1,
    0,
    1, 2, 3, 4, 5, 6, 7,
    1
  ], "error handlers only called if that associated section is read")
})


exports["test ignore error"] = test(function(assert) {
  var boom = Error("Boom!")
  var actual = capture([1, 2, 3, boom], function() {})

  assert(actual, [1, 2, 3], "if nothing returned stream is done")
})

if (require.main === module)
  require("test").run(exports)
});

require.define("/test/signal.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var isReduced = require("../is-reduced")
var flatten = require("../flatten")
var take = require("../take")
var reduce = require("../reduce")
var into = require("../into")
var signal = require("../signal")
var emit = require("../emit")
var close = require("../close")
var when = require("eventual/when")
var isClosed = signal.isClosed
var isOpen = signal.isOpen

exports["test signal basics"] = function(assert, done) {
  var c = signal()

  assert.ok(!signal.isOpen(c), "signal is not open")
  assert.ok(!signal.isClosed(c), "signal is not closed")

  var p = into(c)

  assert.ok(signal.isOpen(c), "signal is open after reduce is called")
  assert.ok(!signal.isClosed(c), "signal is not closed until close is called")

  when(p, function(actual) {
    assert.deepEqual(actual, [ 1, 2, 3, 4 ],
                     "All queued values were accumulated")
    assert.ok(signal.isClosed(c), "signal is closed after it is closed")
    done()
  })

  emit(c, 1)
  emit(c, 2)
  emit(c, 3)
  close(c, 4)
}

exports["test signal auto-close"] = function(assert, done) {
  var c = signal()

  assert.ok(!signal.isOpen(c), "signal is not open")
  assert.ok(!signal.isClosed(c), "signal is not closed")

  var t = take(c, 3)

  assert.ok(!signal.isOpen(c), "signal is not open on take")
  assert.ok(!signal.isClosed(c), "signal is not closed on take")

  var p = into(t)

  assert.ok(signal.isOpen(c), "signal is open after reduce is called")
  assert.ok(!signal.isClosed(c), "signal is not closed until close is called")

  emit(c, 1)
  emit(c, 2)
  emit(c, 3)

  assert.ok(signal.isClosed(c), "signal is closed once consumption is complete")

  var emitresult = emit(c, 4)
  assert.ok(isReduced(emitresult),
            "emit on closed signal returns accumulated")


  var closeresult = close(c)
  assert.ok(isReduced(closeresult),
            "close on closed signal returns accumulated")


  when(p, function(actual) {
    assert.deepEqual(actual, [ 1, 2, 3 ],
                     "All queued values were accumulated")
    assert.ok(signal.isClosed(c), "signal is closed after it is closed")
    done()
  })
}

exports["test signal can have single consumer"] = function(assert, done) {
  var c = signal()
  var p = into(c)

  assert.throws(function() {
    reduce(c, function() { })
  }, "signal can be consumed by a single reader only")

  close(c, 0)
  when(p, function(actual) {
    assert.deepEqual(actual, [ 0 ], "items were accumulated")
    done()
  })
}

/*
exports['test signal'] = function(assert, done) {
  var first = signal()
  var second = signal()

  var actual = into(sequential(flatten([ first, second ])))

  emit(second, 21)
  emit(first, 11)
  emit(first, 12)
  emit(second, 22)
  emit(second, 23)
  close(second)
  close(first)

  go(function(actual) {
    assert.deepEqual(actual, [ 11, 12, 21, 22, 23 ], 'signals preserve order')
    done()
  }, actual)
}

exports['test signal of signals'] = function(assert, done) {
  var owner = signal()
  var first = signal()
  var second = signal()

  var ordered = into(sequential(flatten(owner)))
  var unordered = into(flatten(owner))

  emit(owner, first)
  emit(first, 11)
  emit(owner, second)
  emit(second, 21)
  emit(first, 12)
  emit(second, 22)
  emit(second, 23)
  close(second)
  close(first)
  close(owner)

  go(function(ordered, unordered) {
    assert.deepEqual(ordered, [ 11, 12, 21, 22, 23 ], 'signals preserve order')
    assert.deepEqual(unordered, [ 11, 21, 12, 22, 23 ], 'fifo')
    done()
  }, ordered, unordered)
}

exports['test signal auto closes'] = function(assert, done) {
  var source = signal()
  var subset = pick(3, source)
  var actual = into(subset)

  emit(source, 1)
  emit(source, 2)
  emit(source, 3)

  assert.ok(closed(source), 'signal is closed once consumption is done')

  emit(source, 4)
  emit(source, 5)

  assert.ok(closed(source), 'signal is still closed')

  go(function(actual) {
    assert.deepEqual(actual, [ 1, 2, 3 ], 'no new item were apended')
    done()
  }, actual)
}

exports['test auto close of signal of signals'] = function(assert, done) {
  var owner = signal(0), first = signal(1), second = signal(2)
  var flat = flatten(owner)
  var ordered = into(pick(2, sequential(flat)))
  var unordered = into(pick(4, flat))


  emit(owner, first)
  emit(owner, second)
  emit(second, 21)
  emit(first, 11)
  emit(first, 12)
  assert.ok(!closed(owner), 'signal is still open')
  emit(second, 22)
  emit(second, 23)
  emit(first, 13)

  assert.ok(closed(first), 'first signal is closed')
  assert.ok(closed(second), 'second signal is closed')
  assert.ok(closed(owner), 'owner signal is also closed')

  go(function(ordered, unordered) {
    assert.deepEqual(ordered, [ 11, 12 ], 'no new item were apended')
    assert.deepEqual(unordered, [ 21, 11, 12, 22 ], 'paralleled')
    done()
  }, ordered, unordered)
}
*/

if (module == require.main)
  require("test").run(exports)
});

require.define("/signal.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var accumulate = require("./accumulate")
var reduced = require("./reduced")
var isReduced = require("./is-reduced")
var end = require("./end")
var emit = require("./emit")
var close = require("./close")

var accumulator = "accumulator@" + module.id
var state = "state@" + module.id
var closed = "closed@" + module.id

// Define a `Signal` data-type. A signal is a sequence of "events over time".
// 
// Signals are a building block for creating Reactive event-driven programs.
// 
// If you're familiar with libraries like Node EventEmitter, you might have
// an easy time thinking of Signal as a single event channel. The key difference
// is that signal represents events-over-time as a reducible value. This means
// the events-over-time can be transformed, filtered forked and joined just like
// an array.
function Signal() {}

// Signals can be either open or closed. An open signal may `emit` new values.
// A signal that is not open may not emit.
// 
// Define helper that allow you to test the state of a signal.
function isClosed(signal) {
  return !!signal[closed]
}
function isOpen(signal) {
  return !!signal[accumulator]
}

// Implement accumulate protocol on signals, making them reducible.
accumulate.define(Signal, function(signal, next, initial) {
  // Signals may only be reduced by one consumer function.
  // Other data types built on top of signal may allow for more consumers.
  if (isOpen(signal)) throw Error("Signal is being consumed")
  if (isClosed(signal)) return next(end, initial)
  signal[accumulator] = next
  signal[state] = initial
  return signal
})

emit.define(Signal, function(signal, value) {
  /**
  Emit a new value for signal.
  Throws an exception if the signal is not open for emitting.
  **/
  if (isClosed(signal)) return reduced()
  if (!isOpen(signal)) throw Error("Signal is not open")
  var result = signal[accumulator](value, signal[state])
  if (isReduced(result)) {
    close(signal)
  } else {
    signal[state] = result
  }
  return signal
})

close.define(Signal, function(signal, value) {
  /**
  Close a signal, preventing new values from being emitted.
  Throws an exception if the signal is already closed.
  **/
  if (isClosed(signal)) return reduced()
  if (value !== undefined) emit(signal, value)
  var result = signal[state]
  var next = signal[accumulator]
  signal[closed] = true
  signal[accumulator] = null
  signal[state] = null
  next(end, result)

  return signal
})

// Define a factory function for the `Signal` constructor.
// Assign other signal functions to the factory function object and export
// the result.
function signal() { return new Signal() }
signal.type = Signal
signal.isOpen = isOpen
signal.isClosed = isClosed
signal.emit = emit
signal.close = close

module.exports = signal
});

require.define("/emit.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var Method = require("method")

var emit = Method()

module.exports = emit
});

require.define("/close.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var Method = require("method")


var close = Method()
module.exports = close
});

require.define("/test/hub.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var test = require("./util/test")

var take = require("../take")
var reduce = require("../reduce")
var expand = require("../expand")
var into = require("../into")
var signal = require("../signal")
var emit = require("../emit")
var close = require("../close")
var hub = require("../hub")
var concat = require("../concat")
var delay = require("../delay")
var flatten = require("../flatten")
var capture = require("../capture")
var map = require("../map")
var end = require("../end")

var when = require("eventual/when")

function lazy(f) { return expand(1, f) }

exports["test hub open / close propagate"] = function(assert, done) {
  var c = signal()
  var h = hub(c)

  assert.ok(!signal.isOpen(c), "signal is not open")
  assert.ok(!signal.isClosed(c), "signal is not closed")

  var p = into(h)

  assert.ok(signal.isOpen(c), "signal is open after reduce is called")
  assert.ok(!signal.isClosed(c), "signal is not closed until close is called")

  emit(c, 1)
  emit(c, 2)
  close(c, 3)

  assert.ok(signal.isClosed(c), "signal closed")

  when(p, function(actual) {
    assert.deepEqual(actual, [ 1, 2, 3 ], "all value were propagated")
    done()
  })
}

exports["test multiple subscribtion"] = function(assert, done) {
  var c = signal()
  var h = hub(c)
  var p1 = into(h)

  var p2 = into(h)


  emit(c, 1)

  var p3 = into(h)

  emit(c, 2)
  emit(c, 3)

  var p4 = into(h)

  close(c)

  when(p1, function(actual) {
    assert.deepEqual(actual, [ 1, 2, 3 ], "first consumer get all messages")
    when(p2, function(actual) {
      assert.deepEqual(actual, [ 1, 2, 3],
                       "second consumer get all messages")
      when(p3, function(actual) {
        assert.deepEqual(actual, [ 2, 3 ],
                         "late consumer gets no prior messages")
        when(p4, function(actual) {
          assert.deepEqual(actual, [],
                           "gets no messages if no messages emitd")
          done()
        })
      })
    })
  })
}

exports["test source is closed on end"] = function(assert, done) {
  var c = signal()
  var h = hub(c)
  var t = take(h, 2)

  var p = into(t)

  emit(c, 1)
  emit(c, 2)

  assert.ok(signal.isClosed(c), "signal is closed once consumer is done")

  when(p, function(actual) {
    assert.deepEqual(actual, [ 1, 2 ], "value propagated")
    done()
  })
}

exports["test source is closed on last end"] = function(assert, done) {
  var c = signal()
  var h = hub(c)
  var t1 = take(h, 1)
  var t2 = take(h, 2)
  var t3 = take(h, 3)

  var p1 = into(t1)

  var p2 = into(t2)

  emit(c, 1)

  var p3 = into(t3)

  emit(c, 2)
  emit(c, 3)
  emit(c, 4)

  assert.ok(signal.isClosed(c), "signal is closed once consumer is done")

  when(p1, function(actual) {
    assert.deepEqual(actual, [ 1 ], "#1 took 1 item")
    when(p2, function(actual) {
      assert.deepEqual(actual, [ 1, 2 ], "#2 took 2 items")
      when(p3, function(actual) {
        assert.deepEqual(actual, [ 2, 3, 4 ], "#3 took 3 items")
        done()
      })
    })
  })
}

exports["test reducing closed"] = function(assert, done) {
  var c = signal()
  var h = hub(c)

  var p1 = into(h)

  close(c, 0)
  assert.ok(signal.isClosed(c), "signal is closed")

  var p2 = into(h)

  when(p1, function(actual) {
    assert.deepEqual(actual, [ 0 ], "only item was collected")
    when(p2, function(actual) {
      assert.deepEqual(actual, [], "after close reduces to empty")
      done()
    })
  })
}

exports["test hub with non signals"] = function(assert) {
  assert.equal(hub(null), null, "null is considered to be a hub")
  assert.equal(hub(), undefined, "undefined is considered to be a hub")


  var h = hub([ 1, 2, 3 ])
  var p1 = into(h)
  var p2 = into(h)

  assert.deepEqual(p1, [ 1, 2, 3 ], "first reduce got all items")
  assert.deepEqual(p2, [ 1, 2, 3 ], "second reduce got all items")
}

exports["test hub with sync steam"] = test(function(assert) {
  var source = hub([ 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14 ])

  var s1_3 = take(source, 3)
  var s4_8 = take(source, 5)
  var s9_$ = source

  var actual = concat(s1_3, [ "-" ], s4_8, [ "-" ], s9_$)

  assert(actual, [ 1, 2, 3, "-", 4, 5, 6, 7, 8, "-", 9, 10, 11, 12, 13, 14 ],
         "hub took items in parts")
})

exports["test hub with async stream"] = test(function(assert) {
  var source = hub(delay([1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14 ]))

  var actual = concat(take(source, 3),
                      "-",
                      lazy(function() {
                        return flatten([take(source, 5), take(source, 8)])
                      }),
                      "-",
                      lazy(function() {
                        return source
                      }),
                      "-",
                      lazy(function() {
                        return source
                      }))

  assert(actual, [
    1, 2, 3, "-",
    4, 4, 5, 5, 6, 6, 7, 7, 8, 8, 9, 10, 11, "-",
    12, 13, 14, "-"
  ], "reads in the same turn share head")
})

exports["test hub with broken stream"] = test(function(assert) {
  var boom = Error("boom!")
  var source = hub(concat(delay([1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14 ]),
                   boom))

  var transformed = concat(take(source, 3),
                      "-",
                      lazy(function() {
                        return flatten([take(source, 5), take(source, 8)])
                      }),
                      "-",
                      lazy(function() {
                        return source
                      }),
                      "-",
                      lazy(function() {
                        return source
                      }))
  var actual = capture(transformed, function(error) {
    return error.message
  })


  assert(actual, [
    1, 2, 3, "-",
    4, 4, 5, 5, 6, 6, 7, 7, 8, 8, 9, 10, 11, "-",
    12, 13, 14,
    boom.message
  ], "reads in the same turn share head")

})

exports["test map hub"] = test(function(assert) {
  var called = 0
  var s = signal()
  var h1 = hub(s)
  var m = map(h1, function(x) {
    called = called + 1
    return x
  })
  var h2 = hub(m)

  var actual = concat(into(h2),
                      into(h2),
                      "x",
                      lazy(function() { return called }))

  emit(s, 1)
  emit(s, 2)
  emit(s, 3)
  emit(s, end)

  assert(actual, [ 1, 2, 3, 1, 2, 3, "x", 3 ],
         "hub dispatches on consumers")
})

if (require.main === module)
  require("test").run(exports)
});

require.define("/hub.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var accumulate = require("./accumulate")
var reduced = require("./reduced")
var isReduced = require("./is-reduced")
var end = require("./end")

var input = "input@" + module.id
var consumers = "consumers@" + module.id


var isArray = Array.isArray

function Hub(source) {
  this[input] = source
  this[consumers] = []
}

accumulate.define(Hub, function accumulate(hub, next, initial) {
  // Enqueue new consumer into consumers array so that new
  // values will be delegated to it.
  hub[consumers].push({ next: next, state: initial })
  // If source is not in the process of consumption than
  // start it up.
  if (!isOpen(hub)) open(hub)
})


function drain(consumers) {
  while (consumers.length) {
    var count = consumers.length
    var index = 0
    while (index < count) {
      var consumer = consumers[index]
      consumer.next(end, consumer.state)
      index = index + 1
    }
    consumers.splice(0, count)
  }
}

function dispatch(consumers, value) {
  var count = consumers.length
  var index = 0
  while (index < count) {
    var consumer = consumers[index]
    var state = consumer.next(value, consumer.state)
    // If consumer has finished accumulation remove it from the consumers
    // list. And dispatch end of stream on it (maybe that should not be
    // necessary).
    if (isReduced(state)) {
      consumers.splice(index, 1)
      consumer.next(end, state.value)
      // If consumer is removed than we decrease count as consumers array
      // will contain less elements (unless of course more elements were
      // added but we would like to ignore those).
      count = count - 1
    } else {
      consumer.state = state
      index = index + 1
    }
  }
}

function open(hub) {
  var source = hub[input]
  var reducers = hub[consumers]
  hub[input] = null         // mark hub as open
  accumulate(source, function distribute(value) {
    // If it's end of the source we close all the reducers including
    // ones that subscribe as side effect.
    if (value === end) drain(reducers)
    // otherwise we dispatch value to all the registered reducers.
    else dispatch(reducers, value)

    // reducers will be empty if either source is drained or if all the
    // reducers finished reductions. Either way we reset input back to
    // source and return `reduced` marker to stop the reduction of
    // source.
    if (reducers.length === 0) {
      hub[input] = source
      return reduced()
    }
  })
}

function isOpen(hub) {
  return hub[input] === null
}

function hub(source) {
  /**
  Take a reducible `source`, such as a `signal` and return a reducible that can
  be consumed by many reducers.
  **/
  if (source === null) return null
  if (source === void(0)) return null
  return new Hub(source)
}
hub.isOpen = isOpen
hub.type = Hub

module.exports = hub
});

require.define("/test/queue.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var flatten = require("../flatten")
var take = require("../take")
var reduce = require("../reduce")
var into = require("../into")
var signal = require("../signal")
var emit = require("../emit")
var close = require("../close")
var queue = require("../queue")
var when = require("eventual/when")
var end = require("../end")

exports["test queue before open"] = function(assert, done) {
  var c = signal()
  var q = queue(c)

  emit(q, 1)
  emit(q, 2)

  var p = into(q)

  emit(q, 3)
  emit(q, 4)
  emit(q, end)

  when(p, function(actual) {
    assert.deepEqual(actual, [ 1, 2, 3, 4 ],
                     "queued values were consumed")
    done()
  })
}

if (require.main === module)
  require("test").run(exports)
});

require.define("/queue.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var accumulate = require("./accumulate")
var concat = require("./concat")
var emit = require("./emit")

var queued = "queued@" + module.id
var output = "output@" + module.id

function Queue(target) {
  this[output] = target
  this[queued] = []
}

function isDrained(queue) {
  return !queue[queued]
}

function queue(target) {
  return new Queue(target)
}
queue.type = Queue
queue.emit = function(queue, value) {
  if (isDrained(queue))
    emit(queue[output], value)
  else queue[queued].push(value)
  return queue
}
queue.accumulate = function(queue, next, initial) {
  accumulate(concat(queue[queued], queue[output]), next, initial)
  queue[queued] = null
}

accumulate.define(Queue, queue.accumulate)
emit.define(Queue, queue.emit)

module.exports = queue
});

require.define("/test/buffer.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var into = require("../into")
var reduce = require("../reduce")
var signal = require("../signal")
var emit = require("../emit")
var close = require("../close")
var buffer = require("../buffer")

var when = require("eventual/when")

exports["test signal bufferring"] = function(assert, done) {
  var c = signal()
  var b = buffer(c)

  assert.ok(signal.isOpen(c), "buffer opens a signal")

  emit(c, 1)
  emit(c, 2)

  var p = reduce(b, function(result, value) {
    result.push(value)
    return result
  }, [])

  emit(c, 3)
  close(c, 4)

  when(p, function(actual) {
    assert.deepEqual(actual, [ 1, 2, 3, 4 ],
                     "values have being buffered")
    done()
  })
}

if (require.main === module)
  require("test").run(exports)
});

require.define("/buffer.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var concat = require("./concat")
var hub = require("./hub")
var reducible = require("./reducible")
var end = require("./end")
var accumulate = require("./accumulate")

function push(value, array) {
  array.push(value)
  return array
}

function buffer(input) {
  /**
  Function takes `input` sequence and returns equivalent sequence but buffered.
  So if `input` stream represents result of expensive computation it can be
  buffered to do it only once. Also note that `buffer` is not lazy, it will
  start buffering `input` immediately. For lazy buffering use `cache` instead.
  **/

  // Wrap `input` into the hub, just to be sure that it can be consumed
  // by multiple sources.
  var source = hub(input)
  // Create an array of already buffered values and accumulate source in it.
  var buffered = []
  accumulate(source, push, buffered)
  // Result is a concatenation of buffered values with rest of the source.
  var result = concat(buffered, source)
  return reducible(function reduceReducible(next, initial) {
    // If `end` of stream is already in a buffer, then just reduce it,
    // otherwise reduce concatenation of buffered values and rest of the
    // source.
    return buffered.indexOf(end) >= 0 ? accumulate(buffered, next, initial)
                                      : accumulate(result, next, initial)
  })
}

module.exports = buffer
});

require.define("/test/cache.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var test = require("./util/test")
var lazy = require("./util/lazy")

var concat = require("../concat")
var into = require("../into")
var signal = require("../signal")
var emit = require("../emit")
var cache = require("../cache")
var delay = require("../delay")
var end = require("../end")

var when = require("eventual/when")

exports["test signal cacheing"] = test(function(assert) {
  var s = signal()
  var c = cache(s)

  assert.ok(!signal.isOpen(s), "cache is lazy")
  var value = into(c)
  assert.ok(signal.isOpen(s), "cache reduces input on reduce")


  emit(s, 1)
  emit(s, 2)
  emit(s, 3)
  emit(s, 4)
  emit(s, end)

  assert.ok(signal.isClosed(s), "signal is closed")
  assert.deepEqual(into(value), [ 1, 2, 3, 4 ], "values are dispatched")

  assert(c, [ 1, 2, 3, 4 ], "values are cached")
})

exports["test cache reads once"] = function(assert) {
  var called = 0
  var source = concat(lazy(function() {
                        called = called + 1
                        return 0
                      }),
                      [1, 2, 3])

  var c = cache(source)

  assert.equal(0, called, "cache is lazy")
  assert.deepEqual(into(c), [ 0, 1, 2, 3 ], "values are dispatched")
  assert.deepEqual(into(c), [ 0, 1, 2, 3 ], "second reduce is same")
  assert.equal(1, called, "source was cached")
}

exports["test async cache"] = test(function(assert) {
  var source = delay([1, 2, 3])
  var c = cache(source)

  assert(c, [1, 2, 3], "async streams are cached ok")
})

exports["test async cache reads sync"] = function(assert, done) {
  var source = delay([1, 2, 3])
  var c = cache(source)

  when(into(c), function(actual) {
    assert.deepEqual(actual, [1, 2, 3], "async reads ")
    assert.deepEqual(into(c), [1, 2, 3], "read from cache is sync")
    done()
  })
}

if (require.main === module)
  require("test").run(exports)
});

require.define("/cache.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var buffer = require("./buffer")
var reducible = require("./reducible")
var accumulate = require("./accumulate")

function cache(input) {
  var result
  return reducible(function(next, initial) {
    return result ? accumulate(result, next, initial) :
                    accumulate(result = buffer(input), next, initial)
  })
}

module.exports = cache
});

require.define("/test/pipe.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var test = require("./util/test")
var lazy = require("./util/lazy")

var pipe = require("../pipe")
var concat = require("../concat")
var into = require("../into")
var signal = require("../signal")
var emit = require("../emit")
var cache = require("../cache")
var delay = require("../delay")
var flatten = require("../flatten")


exports["test pipe stream into not open"] = function(assert) {
  var s = signal()
  assert.throws(function() {
    pipe([ 1, 2, 3 ], s)
  }, "can't pipe if can't emit")
}

exports["test pipe multiple streams"] = test(function(assert) {
  var s = signal()
  var actual = into(s)
  pipe([1, 2, 3], s)
  pipe([4, 5, 6], s)


  assert(actual, [1, 2, 3],
         "first end causes close on output all subsequent pipes are ignored")
})

exports["test pipe multiple streams indepenently"] = test(function(assert) {
  var s = signal()
  var actual = into(s)
  pipe(delay([1, 2, 3]), s)
  pipe(delay([4, 5, 6, 7]), s)

  // note that `6` goes through since `end` is also dispatched with a delay.
  assert(actual, [1, 4, 2, 5, 3, 6],
         "parallel pipe works until first end")
})

exports["test pipe multiple streams"] = test(function(assert) {
  var s = signal()
  var actual = into(s)

  pipe(flatten([[1, 2, 3], [4, 5], [ 6, 7 ]]), s)

  // note that `6` goes through since `end` is also dispatched with a delay.
  assert(actual, [1, 2, 3, 4, 5, 6, 7],
         "parallel pipe works until first end")
})

if (require.main === module)
  require("test").run(exports)
});

require.define("/pipe.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var emit = require("./emit")
var close = require("./close")
var end = require("./end")
var accumulate = require("./accumulate")

function pipe(input, output) {
  /**
  Takes `input` sequence and pipes it to the `output` (which is a channel,
  signal or anything that implements `emit`). Note that first end of sequence
  from the piped `input`-s will end an `output` causing subsequent emits to
  fail. If you need to pipe multiple inputs do `pipe(flatten(multiple), output)`
  that way output will close on last end. If you can't flatten all the inputs
  you'll be piping then you can create a signal and pipe flattened version of
  it. That way emitting new input on that channel will automatically pipe all
  it's items into output.
  **/
  accumulate(input, function(value, result) {
    // If signal / channel is closed `accumulated` box is returned
    // that will notify input that consumer is done consuming.
    return value && value.is === end ? close(output, value.value) :
                                       emit(output, value)
  })
}

module.exports = pipe
});

require.define("/test/delay.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var test = require("./util/test")
var concat = require("../concat")
var into = require("../into")
var delay = require("../delay")
var capture = require("../capture")
var lazy = require("./util/lazy")

exports["test delay empty"]  = test(function(assert) {
  var async = false
  var actual = concat(delay([]),
                      lazy(function() { return async }))

  assert(actual, [true], "delay makes things async")
  async = true
})

exports["test sequence of numbers"] = test(function(assert) {
  var async = false
  var actual = concat(delay([1, 2, 3, 4]),
                      lazy(function() { return async }))

  assert(actual, [1, 2, 3, 4, true], "dealy made stream async")
  async = true
})

if (module == require.main)
  require("test").run(exports)
});

require.define("/test/reductions.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var test = require("./util/test")
var lazy = require("./util/lazy")
var delay = require("../delay")
var concat = require("../concat")
var capture = require("../capture")
var reductions = require("../reductions")

exports["test reductions over empty"] = test(function(assert) {
  var called = 0
  var source = reductions([], function(state, x) {
    called = called + 1
    return state + x
  })
  var actual = concat(source,
                      "x",
                      lazy(function() { return called }))

  assert(actual, ["x", 0], "reductions over empty is empty")
})

exports["test reductions over null"] = test(function(assert) {
  var called = 0
  var source = reductions(null, function(state, x) {
    called = called + 1
    return state + x
  }, 0)
  var actual = concat(source,
                      "x",
                      lazy(function() { return called }))

  assert(actual, ["x", 0], "reductions over null is empty")
})

exports["test reductions over void"] = test(function(assert) {
  var called = 0
  var source = reductions(void(0), function(state, x) {
    called = called + 1
    return state + x
  }, 0)
  var actual = concat(source,
                      "x",
                      lazy(function() { return called }))

  assert(actual, ["x", 0], "reductions over void is empty")
})

exports["test reductions over number"] = test(function(assert) {
  var called = 0
  var source = reductions(0, function(state, x) {
    called = called + 1
    return state + x
  }, 0)
  var actual = concat(source,
                      "x",
                      lazy(function() { return called }))

  assert(actual, [0, "x", 1], "reductions over number")
})


exports["test reductions over sequnce"] = test(function(assert) {
  var called = 0
  var source = reductions([1, 1, 1, 1], function(state, x) {
    called = called + 1
    return state + x
  }, 0)
  var actual = concat(source,
                      "x",
                      lazy(function() { return called }))

  assert(actual, [1, 2, 3, 4, "x", 4], "reductions over number")
})

exports["test reductions over async"] = test(function(assert) {
  var called = 0
  var source = reductions(delay([1, 1, 1, 1]), function(state, x) {
    called = called + 1
    return state + x
  }, 0)
  var actual = concat(source,
                      "x",
                      lazy(function() { return called }))

  assert(actual, [1, 2, 3, 4, "x", 4], "reductions over number")
})

exports["test reductions over broken"] = test(function(assert) {
  var called = 0
  var boom = Error("boom")
  var source = reductions(delay([1, 1, 1, 1, boom]), function(state, x) {
    called = called + 1
    return state + x
  }, 0)
  var recovered = capture(source, function(error) { return error.message })
  var actual = concat(recovered,
                      "x",
                      lazy(function() { return called }))

  assert(actual, [1, 2, 3, 4, boom.message, "x", 4], "reductions over broken")
})

if (require.main === module)
  require("test").run(exports)
});

require.define("/reductions.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var reducible = require("./reducible")
var accumulate = require("./accumulate")
var end = require("./end")
var isError = require("./is-error")

function reductions(source, f, initial) {
  /**
  Returns `reducible` collection of the intermediate values of the reduction
  (as per reduce) of `source` by `f`, starting with `initial` value.

  ## Example

  var numbers = reductions([1, 1, 1, 1], function(accumulated, value) {
    return accumulated + value
  }, 0)
  print(numbers) // => < 1 2 3 4 >
  **/
  return reducible(function reduceReducible(next, start) {
    var state = initial
    return accumulate(source, function reduceSource(value, result) {
      if (value === end) return next(end, result)
      if (isError(value)) return next(value, result)
      state = f(state, value)
      return next(state, result)
    }, start)
  })
}

module.exports = reductions
});

require.define("/test/reduce.js",function(require,module,exports,__dirname,__filename,process){"use strict";

var reduce = require("../reduce")
var reduced = require("../reduced")
var into = require("../into")


exports["test reduce"] = function(assert) {
  var result = reduce([ 1, 2, 3 ], function(state, value) {
    return state + value
  }, 0)

  assert.equal(result, 6, "value is accumulated")
}

exports["test reduced early"] = function(assert) {
  var result = reduce([ 1, 2, 3 ], function(state, value) {
    return reduced("nope")
  }, 0)

  assert.equal(result, "nope", "early reduce value is accumulated")
}

exports["test reduce errored"] = function(assert) {
  var boom = Error("Boom!")
  var result = reduce([ 1, 2, boom ], function(state, value) {
    return state + value
  }, 0)

  assert.equal(result, boom, "errors propagate")
}

exports["test reduce late error"] = function(assert) {
  var boom = Error("Boom!")
  var result = reduce([ 1, 2, boom ], function(state, value) {
    return reduced("Wheuh")
  }, 0)

  assert.equal(result, "Wheuh", "late errors are irrelevant")
}

exports["test reduce null"] = function(assert) {
  var result = reduce(null, function(state, value) {
    return state + value
  }, 0)

  assert.equal(result, 0, "null reduces to initial")
}

exports["test reduce void"] = function(assert) {
  var result = reduce(void(0), function(state, value) {
    return state + value
  }, 0)

  assert.equal(result, 0, "void reduces to initial")
}

exports["test reduce empty"] = function(assert) {
  var result = reduce([], function(state, value) {
    return state + value
  }, 0)

  assert.equal(result, 0, "[] reduces to initial")
}

exports["test reduce string"] = function(assert) {
  var result = reduce("world", function(state, value) {
    return state + value
  }, "hello ")

  assert.equal(result, "hello world", "string is equivalent of array of itself")
}

exports["test reduce number"] = function(assert) {
  var result = reduce(7, function(state, value) {
    return state + value
  }, 10)

  assert.equal(result, 17, "number is equivalent of array of itself")
}

exports["test reduce object"] = function(assert) {
  var result = reduce({}, function(state, value) {
    return state + value
  }, "hello ")

  assert.equal(result, "hello " + {},
               "object is equivalent of array of itself")
}

exports["test reduce object"] = function(assert) {
  var boom = Error("Boom!")
  var result = reduce(boom, function(state, value) {
    return state + value
  }, "hello ")

  assert.equal(result, boom, "error is errored collection")
}

if (require.main === module)
  require("test").run(exports)
});

require.define("/test/index.js",function(require,module,exports,__dirname,__filename,process){"use strict";

exports["test filter"] = require("./filter")
exports["test map"] = require("./map")
exports["test take"] = require("./take")
exports["test take while"] = require("./take-while")
exports["test drop"] = require("./drop")
exports["test drop while"] = require("./drop-while")
exports["test into"] = require("./into")
exports["test concat"] = require("./concat")
exports["test flatten"] = require("./flatten")
exports["test zip"] = require("./zip")

exports["test capture"] = require("./capture")

exports["test signal"] = require("./signal")
exports["test hub"] = require("./hub")
exports["test queue"] = require("./queue")
exports["test buffer"] = require("./buffer")
exports["test cache"] = require("./cache")
exports["test pipe"] = require("./pipe")

exports["test delay"] = require("./delay")
exports["test reductions"] = require("./reductions")

exports["test reduce"] = require("./reduce")


require("test").run(exports)
});
require("/test/index.js");
})();

